@@ -1,19 +1,22 @@
 """Activity control nodes."""
 
-import ast
 import math
 
 from gaphas.constraint import constraint
 from gaphas.geometry import Rectangle, distance_line_point
 from gaphas.item import Handle, LinePort
 from gaphas.matrix import Matrix
-from gaphas.util import path_ellipse
 
 from gaphor import UML
 from gaphor.core.modeling import Presentation
 from gaphor.core.modeling.properties import association, attribute, relation_one
-from gaphor.diagram.presentation import ElementPresentation, HandlePositionUpdate, Named
-from gaphor.diagram.shapes import Box, IconBox, Text, stroke
+from gaphor.diagram.presentation import (
+    ElementPresentation,
+    HandlePositionUpdate,
+    Named,
+    literal_eval,
+)
+from gaphor.diagram.shapes import Box, IconBox, Text, ellipse, stroke
 from gaphor.diagram.support import represents
 from gaphor.i18n import gettext
 from gaphor.UML.recipes import stereotypes_str
@@ -57,14 +60,12 @@ class InitialNodeItem(ActivityNodeItem, ElementPresentation):
         self.watch("subject.appliedStereotype.classifier.name")
 
 
-def draw_initial_node(_box, context, _bounding_box):
+def draw_initial_node(_box, context, bounding_box):
     cr = context.cairo
     if stroke := context.style["color"]:
         cr.set_source_rgba(*stroke)
 
-    r = 10
-    d = r * 2
-    path_ellipse(cr, r, r, d, d)
+    ellipse(cr, *bounding_box)
     cr.set_line_width(0.01)
     cr.fill()
 
@@ -104,13 +105,13 @@ def draw_activity_final_node(_box, context, _bounding_box):
     r = outer_radius
 
     d = r * 2
-    path_ellipse(cr, r, r, d, d)
+    ellipse(cr, 0, 0, d, d)
     cr.set_line_width(0.01)
     cr.set_line_width(2)
     stroke(context)
 
     d = inner_radius * 2
-    path_ellipse(cr, r, r, d, d)
+    ellipse(cr, 5, 5, d, d)
     cr.set_line_width(0.01)
     cr.fill()
 
@@ -140,11 +141,11 @@ class FlowFinalNodeItem(ActivityNodeItem, ElementPresentation):
         self.watch("subject.appliedStereotype.classifier.name")
 
 
-def draw_flow_final_node(_box, context, _bounding_box):
+def draw_flow_final_node(_box, context, bounding_box):
     cr = context.cairo
     r = 10
     d = r * 2
-    path_ellipse(cr, r, r, d, d)
+    ellipse(cr, *bounding_box)
     stroke(context)
 
     dr = (1 - math.sin(math.pi / 4)) * r
@@ -221,7 +222,7 @@ class ForkNodeItem(Named, Presentation[UML.ForkNode], HandlePositionUpdate):
         self.watch_handle(h1)
         self.watch_handle(h2)
 
-        self.shape = IconBox(
+        self._shape = IconBox(
             Box(draw=self.draw_fork_node),
             Text(
                 text=lambda: stereotypes_str(self.subject),
@@ -263,14 +264,14 @@ class ForkNodeItem(Named, Presentation[UML.ForkNode], HandlePositionUpdate):
 
     def load(self, name, value):
         if name == "height":
-            self._handles[1].pos.y = ast.literal_eval(value)
+            self._handles[1].pos.y = literal_eval(value)
         else:
             super().load(name, value)
 
     def draw(self, context):
         h1, h2 = self._handles
         height = h2.pos.y - h1.pos.y
-        self.shape.draw(context, Rectangle(0, 0, 1, height))
+        self._shape.draw(context, Rectangle(0, 0, 1, height))
 
     def draw_fork_node(self, _box, context, _bounding_box):
         """
