@@ -3,7 +3,7 @@ import logging
 from gaphor import UML
 from gaphor.core import gettext
 from gaphor.core.modeling.properties import attribute
-from gaphor.core.styling import FontStyle, FontWeight, VerticalAlign
+from gaphor.core.styling import FontStyle, FontWeight, JustifyContent
 from gaphor.diagram.presentation import (
     Classified,
     ElementPresentation,
@@ -11,22 +11,19 @@ from gaphor.diagram.presentation import (
 )
 from gaphor.diagram.shapes import Box, Text, draw_border
 from gaphor.diagram.support import represents
-from gaphor.SysML import sysml
 from gaphor.UML.classes.klass import (
     attribute_watches,
     attributes_compartment,
     operation_watches,
     operations_compartment,
-    stereotype_compartments,
-    stereotype_watches,
 )
+from gaphor.UML.classes.stereotype import stereotype_compartments, stereotype_watches
 
 log = logging.getLogger(__name__)
 
 
 @represents(UML.DataType)
 @represents(UML.PrimitiveType)
-@represents(sysml.ValueType)
 class DataTypeItem(Classified, ElementPresentation[UML.DataType]):
     """This item visualizes a Data Type instance.
 
@@ -54,6 +51,8 @@ class DataTypeItem(Classified, ElementPresentation[UML.DataType]):
     show_operations: attribute[int] = attribute("show_operations", int, default=True)
 
     def additional_stereotypes(self):
+        from gaphor.SysML import sysml
+
         if isinstance(self.subject, UML.PrimitiveType):
             return [gettext("primitive")]
         elif isinstance(self.subject, sysml.ValueType):
@@ -100,7 +99,7 @@ class DataTypeItem(Classified, ElementPresentation[UML.DataType]):
             ),
             *(self.show_stereotypes and stereotype_compartments(self.subject) or []),
             style={
-                "vertical-align": VerticalAlign.TOP,
+                "justify-content": JustifyContent.START,
             },
             draw=draw_border,
         )
