@@ -11,13 +11,12 @@ connected, then dependency type cannot be changed to realization as this
 dependency type can only exist between a component and a classifier.
 
 Function dependency_type in model factory should be used to determine
-type of a dependency in automatic way.
+the type of dependency in an automatic way.
 """
 
-import ast
-
 from gaphor import UML
 from gaphor.core import gettext
+from gaphor.core.modeling.properties import attribute
 from gaphor.diagram.presentation import LinePresentation, Named
 from gaphor.diagram.shapes import Box, Text, stroke
 from gaphor.diagram.support import represents
@@ -26,6 +25,7 @@ from gaphor.UML.recipes import stereotypes_str
 
 
 @represents(UML.Dependency, head=UML.Dependency.supplier, tail=UML.Dependency.client)
+@represents(UML.Usage, head=UML.Usage.supplier, tail=UML.Usage.client)
 class DependencyItem(Named, LinePresentation):
     """Dependency item represents several types of dependencies, i.e. normal
     dependency or usage.
@@ -39,41 +39,35 @@ class DependencyItem(Named, LinePresentation):
     """
 
     def __init__(self, diagram, id=None):
-        super().__init__(diagram, id)
-
-        self._handles[0].pos = (30, 20)
-        self._handles[1].pos = (0, 0)
-
-        self._dependency_type = UML.Dependency
-        # auto_dependency is used by connection logic, not in this class itself
-        self.auto_dependency = True
-
         additional_stereotype = {
             UML.Usage: (gettext("use"),),
             UML.Realization: (gettext("realize"),),
             UML.InterfaceRealization: (gettext("implements"),),
         }
 
-        self.shape_middle = Box(
-            Text(
-                text=lambda: stereotypes_str(
-                    self.subject, additional_stereotype.get(self._dependency_type, ())
+        self._dependency_type = UML.Dependency
+
+        super().__init__(
+            diagram,
+            id,
+            shape_middle=Box(
+                Text(
+                    text=lambda: stereotypes_str(
+                        self.subject,
+                        additional_stereotype.get(self._dependency_type, ()),
+                    ),
                 ),
+                Text(text=lambda: self.subject.name or ""),
             ),
-            Text(text=lambda: self.subject.name or ""),
         )
+
+        self._handles[0].pos = (30, 20)
+        self._handles[1].pos = (0, 0)
+
         self.watch("subject[NamedElement].name")
         self.watch("subject.appliedStereotype.classifier.name")
 
-    def save(self, save_func):
-        super().save(save_func)
-        save_func("auto_dependency", self.auto_dependency)
-
-    def load(self, name, value):
-        if name == "auto_dependency":
-            self.auto_dependency = ast.literal_eval(value)
-        else:
-            super().load(name, value)
+    auto_dependency: attribute[int] = attribute("auto_dependency", int, default=True)
 
     def postload(self):
         if self.subject:
