@@ -1,5 +1,7 @@
 """Generalization --"""
 
+from __future__ import annotations
+
 from gaphor import UML
 from gaphor.core.modeling import DrawContext
 from gaphor.core.modeling.diagram import Diagram
@@ -15,17 +17,21 @@ from gaphor.UML.recipes import stereotypes_str
     tail=UML.Generalization.general,
 )
 class GeneralizationItem(LinePresentation):
-    def __init__(self, diagram: Diagram, id: str = None):
-        super().__init__(diagram, id, style={"dash-style": ()})
+    def __init__(self, diagram: Diagram, id: str | None = None):
+        super().__init__(
+            diagram,
+            id,
+            shape_middle=Box(
+                Text(
+                    text=lambda: stereotypes_str(self.subject),
+                )
+            ),
+            style={"dash-style": ()},
+        )
 
         self._handles[0].pos = (30, 20)
         self._handles[1].pos = (0, 0)
 
-        self.shape_middle = Box(
-            Text(
-                text=lambda: stereotypes_str(self.subject),
-            )
-        )
         self.watch("subject.appliedStereotype.classifier.name")
 
     def draw_tail(self, context: DrawContext):
