@@ -2,9 +2,6 @@
 
 from typing import Optional
 
-from gaphas.connector import ConnectionSink, ItemConnectionSink
-from gaphas.types import Pos, SupportsFloatPos
-
 from gaphor import UML
 from gaphor.core.modeling import Element, Presentation
 from gaphor.diagram.connectors import BaseConnector, Connector
@@ -33,7 +30,7 @@ def get_lifeline(item, handle):
 def order_lifeline_covered_by(lifeline):
     diagram = lifeline.diagram
 
-    def y_and_occurence(connected):
+    def y_and_occurrence(connected):
         for conn in diagram.connections.get_connections(connected=connected):
             m = conn.item.matrix_i2c
             if not conn.item.subject:
@@ -48,7 +45,7 @@ def order_lifeline_covered_by(lifeline):
                     m.transform_point(*conn.item.bottom.pos)[1],
                     conn.item.subject.finish,
                 )
-                yield from y_and_occurence(conn.item)
+                yield from y_and_occurrence(conn.item)
             elif isinstance(conn.item, MessageItem):
                 yield (
                     m.transform_point(*conn.handle.pos)[1],
@@ -58,7 +55,7 @@ def order_lifeline_covered_by(lifeline):
                 )
 
     if lifeline.subject:
-        keys = {o: y for y, o in y_and_occurence(lifeline)}
+        keys = {o: y for y, o in y_and_occurrence(lifeline)}
         lifeline.subject.coveredBy.order(lambda key: keys.get(key, 0.0))
 
 
@@ -108,20 +105,18 @@ def disconnect_lifelines(line, send, received):
     If there are no lifelines connected on both ends, then remove the
     message from the data model.
     """
-    if not line.subject:
+    if not (subject := line.subject):
         return
 
-    if send:
-        if event := line.subject.receiveEvent:
-            event.unlink()
+    if send and (event := subject.sendEvent):
+        event.unlink()
 
-    if received:
-        if event := line.subject.sendEvent:
-            event.unlink()
+    if received and (event := subject.receiveEvent):
+        event.unlink()
 
     # one is disconnected and one is about to be disconnected,
     # so destroy the message
-    if not (send and received):
+    if not (subject.sendEvent or subject.receiveEvent):
         # Both ends are disconnected:
         message = line.subject
         del line.subject
@@ -137,72 +132,43 @@ class MessageLifelineConnect(BaseConnector):
     the lifetime line. In case it's added to the head, the message is
     considered to be part of a communication diagram. If the message is
     added to a lifetime line, it's considered a sequence diagram.
+
+    Message.head has no arrow (sendEvent)
+    Message.tail has the arrow (receiveEvent)
     """
 
     element: LifelineItem
     line: MessageItem
 
-    def allow(self, handle, port):
-        """Glue to lifeline's head or lifetime.
-
-        If lifeline's lifetime is visible then disallow connection to
-        lifeline's head.
-        """
-        element = self.element
-        lifetime = element.lifetime
-        line = self.line
-        opposite = line.opposite(handle)
-
-        ol = self.get_connected(opposite)
-        if isinstance(ol, LifelineItem):
-            opposite_is_visible = ol.lifetime.visible
-            # connect lifetimes if both are visible or both invisible
-            return not (lifetime.visible ^ opposite_is_visible)
-
-        return not (lifetime.visible ^ (port is element.lifetime.port))
-
     def connect(self, handle, port):
         line = self.line
         send = self.get_connected(line.head)
         received = self.get_connected(line.tail)
         connect_lifelines(line, send, received)
-
-        lifetime = self.element.lifetime
-        # if connected to head, then make lifetime invisible
-        if port is lifetime.port:
-            lifetime.min_length = lifetime.MIN_LENGTH_VISIBLE
-        else:
-            lifetime.visible = False
-            lifetime.connectable = False
         return True
 
     def disconnect(self, handle):
         line = self.line
         send: Optional[Presentation[Element]] = get_connected(line, line.head)
         received = self.get_connected(line.tail)
-        lifeline = self.element
-        lifetime = lifeline.lifetime
 
-        # if a message is delete message, then disconnection causes
-        # lifeline to be no longer destroyed (note that there can be
+        # if a message is a deleted message, then the disconnection causes
+        # the lifeline to be no longer destroyed (note that there can be
         # only one delete message connected to lifeline)
         if received and line.subject and line.subject.messageSort == "deleteMessage":
             assert isinstance(received, LifelineItem)
             received.is_destroyed = False
             received.request_update()
 
-        disconnect_lifelines(line, send, received)
-
-        if len(list(self.diagram.connections.get_connections(connected=lifeline))) == 1:
-            # after disconnection count of connected items will be
-            # zero, so allow connections to lifeline's lifetime
-            lifetime.connectable = True
-            lifetime.min_length = lifetime.MIN_LENGTH
+        disconnect_lifelines(
+            line,
+            send and handle is line.head,
+            received and handle is line.tail,
+        )
 
 
 @Connector.register(ExecutionSpecificationItem, MessageItem)
 class ExecutionSpecificationMessageConnect(BaseConnector):
-
     element: ExecutionSpecificationItem
     line: MessageItem
 
@@ -217,18 +183,21 @@ class ExecutionSpecificationMessageConnect(BaseConnector):
         line = self.line
         send = get_lifeline(line, line.head)
         received = get_lifeline(line, line.tail)
-        disconnect_lifelines(line, send, received)
+        disconnect_lifelines(
+            line,
+            send and handle is line.head,
+            received and handle is line.tail,
+        )
 
 
 @Connector.register(LifelineItem, ExecutionSpecificationItem)
 class LifelineExecutionSpecificationConnect(BaseConnector):
-
     element: LifelineItem
     line: ExecutionSpecificationItem
 
     def allow(self, handle, port):
         lifetime = self.element.lifetime
-        return lifetime.visible
+        return super().allow(handle, port) and lifetime.visible
 
     def connect(self, handle, port):
         lifeline = self.element.subject
@@ -238,17 +207,17 @@ class LifelineExecutionSpecificationConnect(BaseConnector):
             exec_spec = model.create(UML.BehaviorExecutionSpecification)
             self.line.subject = exec_spec
 
-            start_occurence: UML.ExecutionOccurrenceSpecification = model.create(
+            start_occurrence: UML.ExecutionOccurrenceSpecification = model.create(
                 UML.ExecutionOccurrenceSpecification
             )
-            start_occurence.covered = lifeline
-            start_occurence.execution = exec_spec
+            start_occurrence.covered = lifeline
+            start_occurrence.execution = exec_spec
 
-            finish_occurence: UML.ExecutionOccurrenceSpecification = model.create(
+            finish_occurrence: UML.ExecutionOccurrenceSpecification = model.create(
                 UML.ExecutionOccurrenceSpecification
             )
-            finish_occurence.covered = lifeline
-            finish_occurence.execution = exec_spec
+            finish_occurrence.covered = lifeline
+            finish_occurrence.execution = exec_spec
 
         if lifeline.interaction:
             exec_spec.enclosingInteraction = lifeline.interaction
@@ -279,7 +248,6 @@ class LifelineExecutionSpecificationConnect(BaseConnector):
 
 @Connector.register(ExecutionSpecificationItem, ExecutionSpecificationItem)
 class ExecutionSpecificationExecutionSpecificationConnect(BaseConnector):
-
     element: ExecutionSpecificationItem
     line: ExecutionSpecificationItem
 
@@ -307,25 +275,3 @@ class ExecutionSpecificationExecutionSpecificationConnect(BaseConnector):
 
         for cinfo in self.diagram.connections.get_connections(connected=self.line):
             Connector(self.line, cinfo.item).disconnect(cinfo.handle)
-
-
-@ConnectionSink.register(LifelineItem)
-class LifelineConnectionSink(ItemConnectionSink):
-    def glue(
-        self, pos: SupportsFloatPos, secondary_pos: Optional[SupportsFloatPos] = None
-    ) -> Optional[Pos]:
-        """Glue a line to the lifeline item, have a preference for the lifetime
-        line."""
-        assert isinstance(self.item, LifelineItem)
-        glue_pos = super().glue(pos, secondary_pos)
-
-        lifetime = self.item.lifetime
-        if lifetime.visible and self.port is not lifetime.port:  # type: ignore[has-type]
-            g, d = lifetime.port.glue(pos)
-            if d < self.distance:
-                self.port = lifetime.port
-                glue_pos = g
-
-            return glue_pos
-
-        return glue_pos
