@@ -6,10 +6,13 @@ Functions collected in this module allow to
 - perform specific searches and manipulations
 """
 
+from __future__ import annotations
+
 import itertools
-from typing import Iterable, Optional, Sequence
+from typing import Iterable, Sequence, TypeVar
 
 from gaphor.UML.uml import (
+    Artifact,
     Association,
     Class,
     Classifier,
@@ -33,13 +36,16 @@ from gaphor.UML.uml import (
     Realization,
     Slot,
     Stereotype,
+    StructuredClassifier,
     Type,
     Usage,
 )
 
+T = TypeVar("T", bound=Element)
+
 
 def stereotypes_str(element: Element, stereotypes: Sequence[str] = ()):
-    """Identify stereotypes of an UML metamodel instance and return coma
+    """Identify stereotypes of a UML metamodel instance and return coma
     separated stereotypes as string.
 
     :Parameters:
@@ -97,16 +103,6 @@ def apply_stereotype(element, stereotype):
     return obj
 
 
-def find_instances(element):
-    """Find instance specification which extend classifier `element`."""
-    model = element.model
-    return model.select(
-        lambda e: e.isKindOf(InstanceSpecification)
-        and e.classifier
-        and e.classifier[0] == element
-    )
-
-
 def remove_stereotype(element, stereotype):
     """Remove a stereotype from an element.
 
@@ -128,7 +124,7 @@ def get_stereotypes(element):
     model = element.model
     # UML specs does not allow to extend stereotypes with stereotypes
     if isinstance(element, Stereotype):
-        return ()
+        return []
 
     cls = type(element)
 
@@ -138,7 +134,13 @@ def get_stereotypes(element):
     # find stereotypes that extend element class
     classes = model.select(lambda e: e.isKindOf(Class) and e.name in names)
 
-    stereotypes = {ext.ownedEnd.type for cls in classes for ext in cls.extension}
+    stereotypes = list({ext.ownedEnd.type for cls in classes for ext in cls.extension})
+
+    for s in stereotypes:
+        for sub in s.specialization[:].specific:
+            if isinstance(sub, Stereotype) and sub not in stereotypes:
+                stereotypes.append(sub)
+
     # Lambda key sort issue in mypy: https://github.com/python/mypy/issues/9656
     return sorted(stereotypes, key=lambda st: st.name)  # type: ignore
 
@@ -270,7 +272,16 @@ def create_connector(type_a: ConnectableElement, type_b: ConnectableElement):
     return conn
 
 
-def set_navigability(assoc, end, nav):
+TYPES_WITH_OWNED_ATTRIBUTE = (
+    Artifact,
+    Class,
+    DataType,
+    Interface,
+    StructuredClassifier,
+)
+
+
+def set_navigability(assoc: Association, end: Property, nav: bool | None) -> None:
     """Set navigability of an association end (property).
 
     There are three possible values for ``nav`` parameter:
@@ -311,11 +322,11 @@ def set_navigability(assoc, end, nav):
 
     When an end is non-navigable, then it is just member of an association.
     """
+    assert end.opposite
+    owner = end.opposite.type
     # remove "navigable" and "unspecified" navigation indicators first
-    if isinstance(end.type, (Class, DataType, Interface)):
-        owner = end.opposite.type
-        if owner and end in owner.ownedAttribute:
-            owner.ownedAttribute.remove(end)
+    if isinstance(owner, TYPES_WITH_OWNED_ATTRIBUTE) and end in owner.ownedAttribute:
+        owner.ownedAttribute.remove(end)
     if end in assoc.ownedEnd:
         assoc.ownedEnd.remove(end)
     if end in assoc.navigableOwnedEnd:
@@ -325,8 +336,7 @@ def set_navigability(assoc, end, nav):
     assert end not in assoc.navigableOwnedEnd
 
     if nav is True:
-        if isinstance(end.type, (Class, DataType, Interface)):
-            owner = end.opposite.type
+        if isinstance(owner, TYPES_WITH_OWNED_ATTRIBUTE):
             owner.ownedAttribute = end
         else:
             assoc.navigableOwnedEnd = end
@@ -351,7 +361,11 @@ def dependency_type(client, supplier):
     # test interface first as it is a classifier
     if isinstance(supplier, Interface):
         dt = Usage
-    elif isinstance(supplier, Component) and isinstance(client, Classifier):
+    elif (
+        isinstance(supplier, Component)
+        and isinstance(client, Classifier)
+        and not isinstance(client, Component)
+    ):
         dt = Realization
 
     return dt
@@ -375,8 +389,7 @@ def clone_message(msg, inverted=False):
         receive.covered = msg.receiveEvent.covered
 
     if inverted:
-        # inverted message goes in different direction, than original
-        # message
+        # inverted message goes in different direction, then the original message
         message.sendEvent = receive
         message.receiveEvent = send
     else:
@@ -385,10 +398,14 @@ def clone_message(msg, inverted=False):
     return message
 
 
-def owner_package(element: Optional[Element]) -> Optional[Package]:
-    if element is None or isinstance(element, Package):
+def owner_of_type(element: Element | None, owner_type: type[T]) -> T | None:
+    if element is None or isinstance(element, owner_type):
         return element
-    return owner_package(element.owner)
+    return owner_of_type(element.owner, owner_type)
+
+
+def owner_package(element: Element | None) -> Package | None:
+    return owner_of_type(element, Package)
 
 
 def swap_element(element, new_class):
