@@ -2,6 +2,8 @@
 clean and in sync with diagrams."""
 
 
+from typing import Iterable
+
 from gaphor import UML
 from gaphor.abc import Service
 from gaphor.core import event_handler
@@ -38,18 +40,14 @@ class SanitizerService(Service):
 
         event_manager.subscribe(self._unlink_on_subject_delete)
         event_manager.subscribe(self.update_annotated_element_link)
-        event_manager.subscribe(self._unlink_on_stereotype_delete)
         event_manager.subscribe(self._unlink_on_extension_delete)
-        event_manager.subscribe(self._disconnect_extension_end)
         event_manager.subscribe(self._redraw_diagram_on_move)
 
     def shutdown(self):
         event_manager = self.event_manager
         event_manager.unsubscribe(self._unlink_on_subject_delete)
         event_manager.unsubscribe(self.update_annotated_element_link)
-        event_manager.unsubscribe(self._unlink_on_stereotype_delete)
         event_manager.unsubscribe(self._unlink_on_extension_delete)
-        event_manager.unsubscribe(self._disconnect_extension_end)
         event_manager.unsubscribe(self._redraw_diagram_on_move)
 
     @event_handler(AssociationSet)
@@ -101,53 +99,15 @@ class SanitizerService(Service):
             comment = comment_item.subject
             comment.annotatedElement = subject
 
-    def perform_unlink_for_instances(self, st, meta):
-        inst = UML.recipes.find_instances(st)
-
-        for i in list(inst):
-            for e in i.extended:
-                if not meta or isinstance(e, meta):
-                    i.unlink()
-
     @event_handler(AssociationDeleted)
     @undo_guard
     def _unlink_on_extension_delete(self, event):
         """Remove applied stereotypes when extension is deleted."""
-        if (
-            isinstance(event.element, UML.Extension)
-            and event.property is UML.Association.memberEnd
-            and event.element.memberEnd
+        if isinstance(event.element, UML.Stereotype) and event.property in (
+            UML.Stereotype.ownedAttribute,
+            UML.Stereotype.generalization,
         ):
-            p = event.element.memberEnd[0]
-            ext = event.old_value
-            if isinstance(p, UML.ExtensionEnd):
-                p, ext = ext, p
-            st = ext.type
-            meta = p.type and getattr(UML, p.type.name, None)
-            if st:
-                self.perform_unlink_for_instances(st, meta)
-
-    @event_handler(AssociationSet)
-    @undo_guard
-    def _disconnect_extension_end(self, event):
-        if event.property is UML.ExtensionEnd.type and event.old_value:
-            ext = event.element
-            p = ext.opposite
-            if not p:
-                return
-            st = event.old_value
-            if st:
-                meta = p.type and getattr(UML, p.type.name, None)
-                self.perform_unlink_for_instances(st, meta)
-
-    @event_handler(AssociationDeleted)
-    @undo_guard
-    def _unlink_on_stereotype_delete(self, event):
-        """Remove applied stereotypes when stereotype is deleted."""
-        if event.property is UML.InstanceSpecification.classifier and isinstance(
-            event.old_value, UML.Stereotype
-        ):
-            event.element.unlink()
+            update_stereotype_application(event.element)
 
     @event_handler(DerivedSet)
     @undo_guard
@@ -156,3 +116,43 @@ class SanitizerService(Service):
             diagram = event.element
             for item in diagram.get_all_items():
                 diagram.request_update(item)
+
+
+def update_stereotype_application(stereotype: UML.Stereotype, seen=None):
+    # Seen is a (mutable) list to avoid infinite loops
+    if seen is None:
+        seen = {stereotype}
+
+    metaclass_names = {m.name for m in metaclasses(stereotype)}
+
+    for instance_spec in list(stereotype.instanceSpecification):
+        for e in list(instance_spec.extended):
+            names = {c.__name__ for c in type(e).__mro__ if issubclass(c, Element)}
+            if names.isdisjoint(metaclass_names):
+                del instance_spec.extended[e]
+        if not instance_spec.extended:
+            instance_spec.unlink()
+
+    for sub_stereotype in stereotype.specialization[:].specific:
+        if sub_stereotype not in seen:
+            seen.add(sub_stereotype)
+            update_stereotype_application(sub_stereotype, seen)
+
+
+def metaclasses(stereotype: UML.Stereotype, seen=None) -> Iterable[UML.Class]:
+    # Seen is a (mutable) list to avoid infinite loops
+    if seen is None:
+        seen = {stereotype}
+
+    extensions = (
+        e
+        for e in stereotype.ownedAttribute[:].association
+        if isinstance(e, UML.Extension)
+    )
+
+    yield from (e.metaclass for e in extensions if e.metaclass)
+
+    for super_stereotype in stereotype.generalization[:].general:
+        if super_stereotype not in seen:
+            seen.add(super_stereotype)
+            yield from metaclasses(super_stereotype, seen)
