@@ -1,12 +1,10 @@
 """Use case diagram item."""
 
-from gaphas.util import path_ellipse
-
 from gaphor import UML
 from gaphor.diagram.presentation import Classified, ElementPresentation
-from gaphor.diagram.shapes import Box, Text, stroke
+from gaphor.diagram.shapes import Box, Text, draw_ellipse
 from gaphor.diagram.support import represents
-from gaphor.diagram.text import FontWeight
+from gaphor.diagram.text import FontStyle, FontWeight
 from gaphor.UML.recipes import stereotypes_str
 
 
@@ -16,27 +14,25 @@ class UseCaseItem(Classified, ElementPresentation):
 
     def __init__(self, diagram, id=None):
         super().__init__(diagram, id, height=30)
+
+        self.watch("subject[NamedElement].name")
+        self.watch("subject.appliedStereotype.classifier.name")
+        self.watch("subject[Classifier].isAbstract", self.update_shapes)
+
+    def update_shapes(self, event=None):
         self.shape = Box(
             Text(
                 text=lambda: stereotypes_str(self.subject),
             ),
             Text(
                 text=lambda: self.subject.name or "",
-                style={"font-weight": FontWeight.BOLD},
+                width=lambda: self.width,
+                style={
+                    "font-weight": FontWeight.BOLD,
+                    "font-style": FontStyle.ITALIC
+                    if self.subject and self.subject.isAbstract
+                    else FontStyle.NORMAL,
+                },
             ),
-            draw=draw_usecase,
+            draw=draw_ellipse,
         )
-
-        self.watch("subject[NamedElement].name")
-        self.watch("subject.appliedStereotype.classifier.name")
-
-
-def draw_usecase(box, context, bounding_box):
-    cr = context.cairo
-
-    rx = bounding_box.width / 2.0
-    ry = bounding_box.height / 2.0
-
-    cr.move_to(bounding_box.width, ry)
-    path_ellipse(cr, rx, ry, bounding_box.width, bounding_box.height)
-    stroke(context)
