@@ -3,4 +3,4 @@ import sys
 from gaphor.ui import main
 
 if __name__ == "__main__":
-    main(sys.argv)
+    sys.exit(main(sys.argv))
