@@ -8,6 +8,7 @@ from __future__ import annotations
 
 import importlib.metadata
 import logging
+from pathlib import Path
 from typing import Iterator, TypeVar, cast
 
 from gaphor import transaction
@@ -54,7 +55,8 @@ class Application(Service, ActionProvider):
     are registered in the "component_registry" service.
     """
 
-    def __init__(self):
+    def __init__(self, gtk_app=None):
+        self._gtk_app = gtk_app
         self._active_session: Session | None = None
         self._sessions: set[Session] = set()
 
@@ -72,6 +74,10 @@ class Application(Service, ActionProvider):
 
         return self._services_by_name[name]
 
+    @property
+    def active_window(self):
+        return self._gtk_app.get_active_window() if self._gtk_app else None
+
     @property
     def sessions(self):
         return self._sessions
@@ -81,6 +87,7 @@ class Application(Service, ActionProvider):
         return self._active_session
 
     def new_session(self, *, filename=None, template=None, services=None, force=False):
+        filename = Path(filename) if filename else None
         if filename is None and template is None:
             return self._new_session(services=services)
 
@@ -125,6 +132,7 @@ class Application(Service, ActionProvider):
         return bool(self._active_session)
 
     def has_session(self, filename):
+        filename = Path(filename)
         return any(
             session for session in self._sessions if session.filename == filename
         )
@@ -193,7 +201,7 @@ class Session(Service):
         )
 
         for name, srv in services_by_name.items():
-            logger.info(f"Initializing service {name}")
+            logger.debug("Initializing service %s", name)
             self.component_registry.register(name, srv)
             self.event_manager.handle(ServiceInitializedEvent(name, srv))
 
@@ -214,11 +222,11 @@ class Session(Service):
 
     def shutdown(self):
         if self.component_registry:
-            for name, srv in reversed(list(self.component_registry.all(Service))):  # type: ignore[misc]
+            for name, srv in reversed(list(self.component_registry.all(Service))):  # type: ignore[type-abstract]
                 self.shutdown_service(name, srv)
 
     def shutdown_service(self, name, srv):
-        logger.info(f"Shutting down service {name}")
+        logger.debug("Shutting down service %s", name)
 
         self.event_manager.handle(ServiceShutdownEvent(name, srv))
         self.component_registry.unregister(srv)
