@@ -10,7 +10,7 @@ import re
 from typing import List
 
 OVERRIDE_RE = re.compile(
-    r"^override\s+(?P<name>[\w.]+)(?:\((?P<derived>[^)]+)\))?\s*(?::\s*(?P<type_hint>[\w\s\[\],\"\| ]+))?$"
+    r"^override\s+(?P<name>[\w.]+)(?:\((?P<derived>[^)]+)\))?\s*(?::\s*(?P<type_hint>[\w\s\[\],\"| ]+))?$"
 )
 
 
@@ -37,7 +37,7 @@ class Overrides:
         line = fp.readline()
         linenum = 1
         while line:
-            if line == "%%\n" or line == "%%":
+            if line in ["%%\n", "%%"]:
                 if lines:
                     bufs.append((list(lines), line_number))
                 line_number = linenum + 1
@@ -58,7 +58,6 @@ class Overrides:
             rest = lines[1:]
             words = line.split()
 
-            # TODO: Create a mech to define dependencies
             if words[0] == "override":
                 m = OVERRIDE_RE.match(line.strip())
                 if not m:
@@ -76,9 +75,7 @@ class Overrides:
             elif words[0] == "header":
                 assert not self.header
                 self.header = "".join(rest)
-            elif words[0] == "comment":
-                pass  # ignore comments
-            else:
+            elif words[0] != "comment":
                 print(f"Unknown word: '{words[0]}', line {line_number:d}")
                 raise SystemExit
 
@@ -88,10 +85,7 @@ class Overrides:
     def get_override(self, key):
         """Write override data for 'key' to a file referred to by 'fp'."""
         _deps, _type_hint, data, line = self.overrides.get(key, ((), None, None, None))
-        if not data:
-            return None
-
-        return f"# {line}{data}"
+        return f"# {line}{data}" if data else None
 
     def get_type(self, key):
         return self.overrides.get(key, (None, "Any"))[1]
