@@ -1,7 +1,4 @@
-"""The Core module provides an entry point for Gaphor's core constructs.
-
-An average module should only need to import this module.
-"""
+"""The Core module provides an entry point for Gaphor's core constructs."""
 
 from gaphor.action import action
 from gaphor.core.eventmanager import event_handler
