@@ -189,7 +189,6 @@ class attribute(umlproperty, Generic[T]):
     Element.attr = attribute('attr', types.StringType, '')
     """
 
-    # TODO: check if lower and upper are actually needed for attributes
     def __init__(
         self,
         name: str,
@@ -204,10 +203,10 @@ class attribute(umlproperty, Generic[T]):
         """Load the attribute value."""
         try:
             setattr(obj, self._name, self.type(value))
-        except ValueError:
+        except ValueError as e:
             error_msg = f"Failed to load attribute {self._name} of type {self.type} with value {value}"
 
-            raise TypeError(error_msg)
+            raise TypeError(error_msg) from e
 
     def unlink(self, obj):
         self.set(obj, self.default)
@@ -340,15 +339,14 @@ class association(umlproperty):
                 save_func(self.name, v)
 
     def load(self, obj, value):
-        if not isinstance(value, self.type):
-            raise TypeError(
-                f"Value for {self.name} should be of type {self.type.__name__} ({type(value).__name__})"
-            )
+        if self.opposite:
+            # Loading should not steal references from other elements
+            opposite = getattr(type(value), self.opposite)
+            if opposite.upper == 1 and opposite.get(value):
+                log.debug(f"Cannot steal reference from {value}")
+                return
 
-        if self.upper == 1:
-            self._set_one(obj, value)
-        else:
-            self._set_many(obj, value, from_load=True)
+        self.set(obj, value)
 
     def __str__(self):
         if self.lower == self.upper:
@@ -368,7 +366,7 @@ class association(umlproperty):
     def _get_many(self, obj) -> collection[T]:
         v: collection[T] | None = getattr(obj, self._name, None)
         if v is None:
-            # Create the empty collection here since it might
+            # Create the empty collection here since it may
             # be used to add.
             v = collection(self, obj, self.type)
             setattr(obj, self._name, v)
@@ -402,7 +400,7 @@ class association(umlproperty):
             return
 
         if old:
-            self.delete(obj, old, from_opposite=from_opposite, do_notify=False)
+            self._del_one(obj, old, do_notify=False)
 
         if value is not None:
             setattr(obj, self._name, value)
@@ -430,7 +428,8 @@ class association(umlproperty):
         try:
             self._set_opposite(obj, value, from_opposite)
         except Exception:
-            c.items.pop()
+            if value in c.items:
+                c.items.remove(value)
             raise
 
         self.handle(AssociationAdded(obj, self, value))
@@ -445,7 +444,7 @@ class association(umlproperty):
             if not self.stub:
                 self.stub = associationstub(self)
                 # Do not let property start with underscore, or it will not be found
-                # as a umlproperty.
+                # as an umlproperty.
                 setattr(self.type, "GAPHOR__associationstub__%x" % id(self), self.stub)
             self.stub.set(value, obj)
 
@@ -478,9 +477,9 @@ class association(umlproperty):
 
         self._del_opposite(obj, value, from_opposite)
 
-        c: collection = self._get_many(obj)
-        if c:
-            items = c.items
+        c: collection
+        if c := self._get_many(obj):
+            items: collectionlist = c.items
             try:
                 items.remove(value)
             except ValueError:
@@ -501,14 +500,12 @@ class association(umlproperty):
                 self.stub.delete(value, obj, from_opposite=True)
 
     def unlink(self, obj):
-        values = self.get(obj)
-        composite = self.composite
-        if values:
+        if values := self.get(obj):
             if self.upper == 1:
                 values = [values]
 
+            composite = self.composite
             for value in list(values):
-                # TODO: make normal unlinking work through this method.
                 self.delete(obj, value)
                 if composite:
                     value.unlink()
@@ -520,7 +517,7 @@ class AssociationStubError(Exception):
 
 class associationstub(umlproperty):
     """An association stub is an internal thingy that ensures all associations
-    are always bi-directional.
+    are always bidirectional.
 
     This helps the application when one end of the association is
     unlinked. On unlink() of an element all `umlproperty`'s are iterated
@@ -642,7 +639,7 @@ class derived(umlproperty, Generic[T]):
     def _update(self, obj):
         """Update the list of items.
 
-        Returns a unioncache instance.
+        Returns an unioncache instance.
         """
         u = self.filter(obj)
         if self.upper == 1:
@@ -677,14 +674,9 @@ class derived(umlproperty, Generic[T]):
     def propagate(self, event):
         """Re-emit state change for the derived properties as Derived*Event's.
 
-        TODO: We should fetch the old and new state of the namespace item in
-        stead of the old and new values of the item that changed.
-
-        If multiplicity is [0..1]:
-          send DerivedSet if len(union) < 2
-        if multiplicity is [*]:
-          send DerivedAdded and DerivedDeleted
-            if value not in derived union and
+        If multiplicity is [0..1]:   send DerivedSet if len(union) < 2
+        if multiplicity is [*]:   send DerivedAdded and DerivedDeleted
+        if value not in derived union and
         """
         if event.property not in self.subsets:
             return
@@ -693,11 +685,6 @@ class derived(umlproperty, Generic[T]):
 
         # mimic the events for Set/Add/Delete
         if self.upper == 1:
-            # This is a [0..1] event
-            # TODO: This is an error: [0..*] associations may be used for updating [0..1] associations
-            # assert isinstance(
-            #     event, AssociationSet
-            # ), f"Can only handle [0..1] set-events, not {event} for {event.element}"
             old_value = hasattr(event.element, self._name) and self.get(event.element)
             # Make sure unions are created again
             self.version += 1
@@ -760,7 +747,7 @@ class derivedunion(derived[T]):
             if s is exclude or not object_has_property(obj, s):
                 continue
 
-            tmp = s.__get__(obj)
+            tmp: Iterable[T] | T | None = s.get(obj)
             if isinstance(tmp, Iterable):
                 u.update(tmp)
             elif tmp:
@@ -771,14 +758,9 @@ class derivedunion(derived[T]):
     def propagate(self, event):
         """Re-emit state change for the derived union (as Derived*Event's).
 
-        TODO: We should fetch the old and new state of the namespace item in
-        stead of the old and new values of the item that changed.
-
-        If multiplicity is [0..1]:
-          send DerivedSet if len(union) < 2
-        if multiplicity is [*]:
-          send DerivedAdded and DerivedDeleted
-            if value not in derived union and
+        If multiplicity is [0..1]:   send DerivedSet if len(union) < 2
+        if multiplicity is [*]:   send DerivedAdded and DerivedDeleted
+        if value not in derived union and
         """
         if event.property not in self.subsets:
             return
