@@ -2,7 +2,7 @@
 
 Original module: cssselect2.parser
 :copyright: (c) 2012 by Simon Sapin, 2017 by Guillaume Ayoub.
-:license: BSD, see LICENSE for more details.
+:license: BSD, see https://github.com/Kozea/cssselect2/blob/master/LICENSE for more details.
 """
 
 from tinycss2 import parse_component_value_list
@@ -29,7 +29,7 @@ def parse(input, namespaces=None):
         elif next == ",":
             yield parse_selector(tokens, namespaces)
         else:
-            raise SelectorError(next, "unpexpected %s token." % next.type)
+            raise SelectorError(next, f"unpexpected {next.type} token.")
 
 
 def parse_selector(tokens, namespaces):
@@ -63,8 +63,7 @@ def parse_compound_selector(tokens, namespaces):
         return CompoundSelector(simple_selectors)
     peek = tokens.peek()
     raise SelectorError(
-        peek,
-        "expected a compound selector, got %s" % (peek.type if peek else "EOF"),
+        peek, f'expected a compound selector, got {peek.type if peek else "EOF"}'
     )
 
 
@@ -94,7 +93,7 @@ def parse_simple_selector(tokens, namespaces):
         tokens.next()
         next = tokens.next()
         if next is None or next.type != "ident":
-            raise SelectorError(next, "Expected a class name, got %s" % next)
+            raise SelectorError(next, f"Expected a class name, got {next}")
         return ClassSelector(next.value)
     elif peek.type == "[] block":
         tokens.next()
@@ -105,16 +104,14 @@ def parse_simple_selector(tokens, namespaces):
         if next == ":":
             next = tokens.next()
             if next is None or next.type != "ident":
-                raise SelectorError(
-                    next, "Expected a pseudo-element name, got %s" % next
-                )
+                raise SelectorError(next, f"Expected a pseudo-element name, got {next}")
             return PseudoElementSelector(next.lower_value)
         elif next is not None and next.type == "ident":
             return PseudoClassSelector(next.lower_value)
         elif next is not None and next.type == "function":
             return FunctionalPseudoClassSelector(next.lower_name, next.arguments)
         else:
-            raise SelectorError(next, "unexpected %s token." % next)
+            raise SelectorError(next, f"unexpected {next} token.")
     else:
         return None
 
@@ -124,7 +121,7 @@ def parse_attribute_selector(tokens, namespaces):
     qualified_name = parse_qualified_name(tokens, namespaces, is_attribute=True)
     if qualified_name is None:
         next = tokens.next()
-        raise SelectorError(next, "expected attribute name, got %s" % next)
+        raise SelectorError(next, f"expected attribute name, got {next}")
     namespace, local_name = qualified_name
 
     # Allow syntax like "subject.ownedAttribute"
@@ -157,15 +154,15 @@ def parse_attribute_selector(tokens, namespaces):
         next = tokens.next()
         if next is None or next.type not in ("ident", "string"):
             next_type = "None" if next is None else next.type
-            raise SelectorError(next, "expected attribute value, got %s" % next_type)
+            raise SelectorError(next, f"expected attribute value, got {next_type}")
         value = next.value
     else:
-        raise SelectorError(peek, "expected attribute selector operator, got %s" % peek)
+        raise SelectorError(peek, f"expected attribute selector operator, got {peek}")
 
     tokens.skip_whitespace()
     next = tokens.next()
     if next is not None:
-        raise SelectorError(next, "expected ], got %s" % next.type)
+        raise SelectorError(next, f"expected ], got {next.type}")
     return AttributeSelector(namespace, local_name, operator, value)
 
 
@@ -189,14 +186,15 @@ def parse_qualified_name(tokens, namespaces, is_attribute=False):
         namespace = namespaces.get(first_ident.value)
         if namespace is None:
             raise SelectorError(
-                first_ident, "undefined namespace prefix: " + first_ident.value
+                first_ident, f"undefined namespace prefix: {first_ident.value}"
             )
+
     elif peek == "*":
         next = tokens.next()
         peek = tokens.peek()
         if peek != "|":
             if is_attribute:
-                raise SelectorError(next, "Expected local name, got %s" % next.type)
+                raise SelectorError(next, f"Expected local name, got {next.type}")
             return namespaces.get(None, None), None
         tokens.next()
         namespace = None
@@ -213,23 +211,20 @@ def parse_qualified_name(tokens, namespaces, is_attribute=False):
     elif next == "*" and not is_attribute:
         return namespace, None
     else:
-        raise SelectorError(next, "Expected local name, got %s" % next.type)
+        raise SelectorError(next, f"Expected local name, got {next.type}")
 
 
 class SelectorError(ValueError):
     """A specialized ``ValueError`` for invalid selectors."""
 
 
-class TokenStream(object):
+class TokenStream:
     def __init__(self, tokens):
         self.tokens = iter(tokens)
         self.peeked = []  # In reversed order
 
     def next(self):
-        if self.peeked:
-            return self.peeked.pop()
-        else:
-            return next(self.tokens, None)
+        return self.peeked.pop() if self.peeked else next(self.tokens, None)
 
     def peek(self):
         if not self.peeked:
@@ -256,7 +251,7 @@ class TokenStream(object):
         return self.skip(["comment", "whitespace"])
 
 
-class CombinedSelector(object):
+class CombinedSelector:
     def __init__(self, left, combinator, right):
         #: Combined or compound selector
         self.left = left
@@ -275,7 +270,7 @@ class CombinedSelector(object):
         return "{!r}{}{!r}".format(self.left, self.combinator, self.right)
 
 
-class CompoundSelector(object):
+class CompoundSelector:
     """Aka.
 
     sequence of simple selectors, in Level 3.
@@ -297,7 +292,7 @@ class CompoundSelector(object):
         return "".join(map(repr, self.simple_selectors))
 
 
-class LocalNameSelector(object):
+class LocalNameSelector:
     specificity = 0, 0, 1
 
     def __init__(self, local_name):
@@ -307,7 +302,7 @@ class LocalNameSelector(object):
         return self.local_name
 
 
-class NamespaceSelector(object):
+class NamespaceSelector:
     specificity = 0, 0, 0
 
     def __init__(self, namespace):
@@ -316,33 +311,30 @@ class NamespaceSelector(object):
         self.namespace = namespace
 
     def __repr__(self):
-        if self.namespace == "":
-            return "|"
-        else:
-            return "{%s}|" % self.namespace
+        return "|" if self.namespace == "" else "{%s}|" % self.namespace
 
 
-class IDSelector(object):
+class IDSelector:
     specificity = 1, 0, 0
 
     def __init__(self, ident):
         self.ident = ident
 
     def __repr__(self):
-        return "#" + self.ident
+        return f"#{self.ident}"
 
 
-class ClassSelector(object):
+class ClassSelector:
     specificity = 0, 1, 0
 
     def __init__(self, class_name):
         self.class_name = class_name
 
     def __repr__(self):
-        return "." + self.class_name
+        return f".{self.class_name}"
 
 
-class AttributeSelector(object):
+class AttributeSelector:
     specificity = 0, 1, 0
 
     def __init__(self, namespace, name, operator, value):
@@ -365,7 +357,7 @@ class PseudoClassSelector:
         self.name = name
 
     def __repr__(self):
-        return ":" + self.name
+        return f":{self.name}"
 
 
 class PseudoElementSelector:
@@ -375,10 +367,10 @@ class PseudoElementSelector:
         self.name = name
 
     def __repr__(self):
-        return "::" + self.name
+        return f"::{self.name}"
 
 
-class FunctionalPseudoClassSelector(object):
+class FunctionalPseudoClassSelector:
     specificity = 0, 1, 0
 
     def __init__(self, name, arguments):
