@@ -1,5 +1,8 @@
-"""Compiler for CSS selectors, based on cssselect2.compiler, written by Simon
-Sapin and Guillaume Ayoub."""
+"""Compiler for CSS selectors.
+
+Based on cssselect2.compiler, written by Simon Sapin and Guillaume
+Ayoub.
+"""
 
 import re
 from functools import singledispatch
@@ -27,7 +30,7 @@ def compile_selector_list(input):
 def compile_node(selector):
     """Dynamic dispatch selector nodes.
 
-    Default behavior is a deny (no match).
+    Default behavior is to deny (no match).
     """
     raise parser.SelectorError("Unknown selector", selector)
 
@@ -44,8 +47,7 @@ def compile_name_selector(selector: parser.LocalNameSelector):
 
 
 def ancestors(el):
-    p = el.parent()
-    if p:
+    if p := el.parent():
         yield p
         yield from ancestors(p)
 
@@ -99,7 +101,7 @@ def compile_attribute_selector(selector: parser.AttributeSelector):
 
         def pipe_equal_matcher(el):
             v = el.attribute(name)
-            return v == value or (v and v.startswith(value + "-"))
+            return v == value or v and v.startswith(f"{value}-")
 
         return pipe_equal_matcher
     else:
