@@ -8,7 +8,6 @@ import abc
 from typing import Callable, Dict, Iterable, List, Tuple, Type
 
 import gaphas.item
-from gaphas.decorators import g_async
 from gaphas.segment import Segment
 from gi.repository import Gtk
 
@@ -112,12 +111,6 @@ class EditableTreeModel(Gtk.ListStore):
     def on_row_inserted(self, model, path, iter):
         """This method is called when new elements are added and when a row is
         moved via DnD."""
-        self._sync_model()
-
-    @g_async(single=True)
-    def _sync_model(self):
-        """Align the order of elements in the model with the order in the list
-        store."""
         new_order = [row[-1] for row in self if row[-1]]
         self.sync_model(new_order)
 
@@ -187,8 +180,7 @@ class EditableTreeModel(Gtk.ListStore):
 
     def remove(self, iter):
         """Remove object from GTK model and destroy it."""
-        obj = self._get_object(iter)
-        if obj:
+        if obj := self._get_object(iter):
             obj.unlink()
             return super().remove(iter)
         else:
@@ -268,6 +260,36 @@ def combo_box_text_auto_complete(
         entry.set_text(text)
 
 
+def help_link(builder, help_widget, popover):
+    """Show the help popover for a `Help` link in the property page.
+
+    GTK4 only.
+    """
+
+    def on_activate(*_args):
+        builder.get_object(popover).show()
+
+    help = builder.get_object(help_widget)
+    help.set_accessible_role(Gtk.AccessibleRole.BUTTON)
+    click_handler = Gtk.GestureClick.new()
+    click_handler.connect("released", on_activate)
+    help.add_controller(click_handler)
+
+
+def handler_blocking(widget, event_name, widget_handler):
+    changed_id = widget.connect(event_name, widget_handler)
+
+    def handler_wrapper(func):
+        def _handler(event):
+            widget.handler_block(changed_id)
+            func(event)
+            widget.handler_unblock(changed_id)
+
+        return _handler
+
+    return handler_wrapper
+
+
 @PropertyPages.register(gaphas.item.Line)
 class LineStylePage(PropertyPageBase):
     """Basic line style properties: color, orthogonal, etc."""
@@ -339,13 +361,10 @@ class NotePropertyPage(PropertyPageBase):
             buffer.set_text(subject.note)
         text_view.set_buffer(buffer)
 
-        changed_id = buffer.connect("changed", self._on_body_change)
-
+        @handler_blocking(buffer, "changed", self._on_body_change)
         def handler(event):
             if not text_view.props.has_focus:
-                buffer.handler_block(changed_id)
                 buffer.set_text(event.new_value or "")
-                buffer.handler_unblock(changed_id)
 
         self.watcher.watch("note", handler)
         text_view.connect("destroy", self.watcher.unsubscribe_all)
