@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import math
 from dataclasses import replace
 from math import pi
 from typing import Callable
@@ -7,7 +8,13 @@ from typing import Callable
 from gaphas.geometry import Rectangle
 
 from gaphor.core.modeling import DrawContext, UpdateContext
-from gaphor.core.styling import Style, TextAlign, VerticalAlign, merge_styles
+from gaphor.core.styling import (
+    JustifyContent,
+    Style,
+    TextAlign,
+    VerticalAlign,
+    merge_styles,
+)
 from gaphor.diagram.text import Layout
 
 
@@ -76,6 +83,30 @@ def draw_top_separator(box: Box, context: DrawContext, bounding_box: Rectangle):
     stroke(context, fill=False)
 
 
+def draw_ellipse(box: Box, context: DrawContext, bounding_box: Rectangle):
+    ellipse(context.cairo, *bounding_box)
+
+    stroke(context)
+
+
+def ellipse(cr, x, y, w, h, dc=(4 / 3) * math.tan(math.pi / 8)):
+    rx = x + w / 2.0
+    ry = y + h / 2.0
+    x1 = x + w
+    y1 = y + h
+    tw = w * dc / 2
+    th = h * dc / 2
+
+    # curve: control_a, control_b, end
+    cr.move_to(x, ry)
+    cr.curve_to(x, ry - th, rx - tw, y, rx, y)
+    cr.curve_to(rx + tw, y, x1, ry - th, x1, ry)
+    cr.curve_to(x1, ry + th, rx + tw, y1, rx, y1)
+    cr.curve_to(rx - tw, y1, x, ry + th, x, ry)
+
+    cr.close_path()
+
+
 class Box:
     """A box like shape.
 
@@ -90,9 +121,11 @@ class Box:
     def __init__(
         self,
         *children,
-        style: Style = {},
+        style: Style | None = None,
         draw: Callable[[Box, DrawContext, Rectangle], None] | None = None,
     ):
+        if style is None:
+            style = {}
         self.children = children
         self.sizes: list[tuple[int, int]] = []
         self._inline_style = style
@@ -132,26 +165,74 @@ class Box:
         style = merge_styles(context.style, self._inline_style)
         new_context = replace(context, style=style)
         padding_top, padding_right, padding_bottom, padding_left = style["padding"]
-        valign = style.get("vertical-align", VerticalAlign.MIDDLE)
-        height = sum(h for _w, h in self.sizes)
+        sizes = self.sizes
 
-        if self._draw_border:
-            self._draw_border(self, new_context, bounding_box)
-        x = bounding_box.x + padding_left
-        if valign is VerticalAlign.MIDDLE:
+        justify_content = style.get("justify-content")
+        if not justify_content:
+            justify_content = {
+                VerticalAlign.TOP: JustifyContent.START,
+                VerticalAlign.MIDDLE: JustifyContent.CENTER,
+                VerticalAlign.BOTTOM: JustifyContent.END,
+            }[style.get("vertical-align", VerticalAlign.MIDDLE)]
+
+        if justify_content is JustifyContent.STRETCH and sizes:
+            height = bounding_box.height
+            avg_height = height / len(sizes)
+            oversized = [h for _w, h in sizes if h > avg_height]
+            d = len(sizes) - len(oversized)
+            avg_height = (height - sum(oversized)) / d if d > 0 else 0
+        else:
+            height = sum(h for _w, h in sizes)
+            avg_height = 0
+
+        if justify_content is JustifyContent.CENTER:
             y = (
                 bounding_box.y
                 + padding_top
                 + (max(height, bounding_box.height - padding_top) - height) / 2
             )
-        elif valign is VerticalAlign.BOTTOM:
+        elif justify_content is JustifyContent.END:
             y = bounding_box.y + bounding_box.height - height - padding_bottom
         else:
             y = bounding_box.y + padding_top
+
+        if self._draw_border:
+            self._draw_border(self, new_context, bounding_box)
+
+        x = bounding_box.x + padding_left
         w = bounding_box.width - padding_right - padding_left
-        for c, (_w, h) in zip(self.children, self.sizes):
-            c.draw(context, Rectangle(x, y, w, h))
-            y += h
+        if self.children:
+            last_child = self.children[-1]
+            for c, (_w, h) in zip(self.children, sizes):
+                if c is last_child and justify_content is JustifyContent.START:
+                    h = bounding_box.height - y
+                elif h < avg_height:
+                    h = avg_height
+                c.draw(context, Rectangle(x, y, w, h))
+                y += h
+
+
+class BoundedBox(Box):
+    """A Box.
+
+    It keeps track of the latest bounding box used for drawing.
+    """
+
+    def __init__(
+        self,
+        *children,
+        style: Style = {},
+        draw: Callable[[Box, DrawContext, Rectangle], None] | None = None,
+    ):
+        super().__init__(*children, style=style, draw=draw)
+        self.bounding_box = Rectangle()
+
+    def draw(self, context: DrawContext, bounding_box: Rectangle):
+        self.bounding_box = bounding_box
+        return super().draw(context, bounding_box)
+
+    def __contains__(self, pos_or_rect):
+        return pos_or_rect in self.bounding_box
 
 
 class IconBox:
@@ -232,7 +313,9 @@ class IconBox:
 
 
 class Text:
-    def __init__(self, text=lambda: "", width=lambda: -1, style: Style = {}):
+    def __init__(self, text=lambda: "", width=lambda: -1, style: Style | None = None):
+        if style is None:
+            style = {}
         self._text = text if callable(text) else lambda: text
         self.width = width if callable(width) else lambda: width
         self._inline_style = style
