@@ -2,10 +2,10 @@ from gaphas.view import GtkView
 from gi.repository import Gdk, Gtk
 
 from gaphor.core import Transaction
-from gaphor.core.modeling import Presentation
+from gaphor.core.modeling import Presentation, self_and_owners
 
 
-def shortcut_tool(view, modeling_language, event_manager):
+def shortcut_tool(view, event_manager):
     if Gtk.get_major_version() == 3:
         ctrl = Gtk.EventControllerKey.new(view)
     else:
@@ -21,9 +21,7 @@ def on_delete(ctrl, keyval, keycode, state, event_manager):
     otherwise this key will confuse the text edit stuff.
     """
     view: GtkView = ctrl.get_widget()
-    if keyval in (Gdk.KEY_Delete, Gdk.KEY_BackSpace) and (
-        state == 0 or state & Gdk.ModifierType.MOD2_MASK
-    ):
+    if keyval in (Gdk.KEY_Delete, Gdk.KEY_BackSpace):
         delete_selected_items(view, event_manager)
         return True
     return False
@@ -33,8 +31,7 @@ def delete_selected_items(view: GtkView, event_manager):
     with Transaction(event_manager):
         items = view.selection.selected_items
         for i in list(items):
-            if isinstance(i, Presentation):
-                i.unlink()
-            else:
-                if i.diagram:
-                    i.diagram.remove(i)
+            assert isinstance(i, Presentation)
+            if i.subject and i.subject in self_and_owners(i.diagram):
+                del i.diagram.element
+            i.unlink()
