@@ -7,10 +7,8 @@ import sphinx.util.docutils
 from docutils import nodes
 from docutils.parsers.rst import directives
 from docutils.parsers.rst.directives import images
-from sphinx.ext.autodoc.mock import mock
 from sphinx.util import logging
 
-from gaphor.core.eventmanager import EventManager
 from gaphor.core.modeling import Diagram, ElementFactory
 from gaphor.diagram.export import save_pdf, save_svg
 from gaphor.i18n import gettext
@@ -70,11 +68,7 @@ class DiagramDirective(sphinx.util.docutils.SphinxDirective):
         self.env.note_dependency(model_file)
         model = load_model(Path(self.env.srcdir) / model_file)
 
-        outdir = (
-            (Path(self.env.app.doctreedir) / ".." / "gaphor")
-            .resolve()
-            .relative_to(self.env.srcdir)
-        )
+        outdir = (Path(self.env.app.doctreedir) / ".." / "gaphor").resolve()
         outdir.mkdir(exist_ok=True)
 
         diagram = next(
@@ -101,15 +95,16 @@ class DiagramDirective(sphinx.util.docutils.SphinxDirective):
         save_pdf(outfile.with_suffix(".pdf"), diagram)
 
         # Image needs a relative path. Make our outfile path relative to the doc
+        outdir = outdir.relative_to(self.env.srcdir)
+        outfile = outdir / f"{diagram.id}"
+
         for _ in Path(self.env.docname).parts[:-1]:
             outfile = Path("..") / outfile
 
         return [
             nodes.image(
-                rawsource=self.block_text,
-                uri=str(outfile) + ".*",
-                **self.options,
-            ),
+                rawsource=self.block_text, uri=f"{str(outfile)}.*", **self.options
+            )
         ]
 
     def logging_error_node(self, text: str) -> list[nodes.Node]:
@@ -122,12 +117,12 @@ class DiagramDirective(sphinx.util.docutils.SphinxDirective):
 def load_model(model_file: str) -> ElementFactory:
     element_factory = ElementFactory()
 
-    with mock(["gi.repository.Gtk", "gi.repository.Gdk"]):
-        modeling_language = ModelingLanguageService(EventManager())
+    modeling_language = ModelingLanguageService()
 
-    storage.load(
-        model_file,
-        element_factory,
-        modeling_language,
-    )
+    with open(model_file) as file_obj:
+        storage.load(
+            file_obj,
+            element_factory,
+            modeling_language,
+        )
     return element_factory
