@@ -13,6 +13,7 @@
 import code
 import sys
 import textwrap
+import traceback
 from typing import Dict, List
 
 import jedi
@@ -64,7 +65,9 @@ def docstring_dedent(docstr: str) -> str:
 
 
 class Help:
-    def __init__(self, writer, locals={}):
+    def __init__(self, writer, locals=None):
+        if locals is None:
+            locals = {}
         self._writer = writer
         self._locals = locals
 
@@ -99,7 +102,7 @@ class TextViewWriter:
     """A Multiplexing output stream.
 
     It can replace another stream, and tee output to the original stream
-    and too a GTK textview.
+    and to a GTK textview.
     """
 
     def __init__(self, name, view):
@@ -120,6 +123,27 @@ class TextViewWriter:
         setattr(sys, self.name, self.out)
 
 
+class ExceptionWriter:
+    """A Multiplexing output stream.
+
+    It can replace another stream, and tee output to the original stream
+    and to a GTK textview.
+    """
+
+    def excepthook(self, exc_type, exc_value, exc_traceback):
+        traceback.print_exception(
+            exc_type, value=exc_value, tb=exc_traceback, file=sys.stderr
+        )
+
+    def __enter__(self):
+        self.orig_excepthook = sys.excepthook
+        sys.excepthook = self.excepthook
+        return self
+
+    def __exit__(self, exception_type, exception_value, traceback):
+        sys.excepthook = self.orig_excepthook
+
+
 class GTKInterpreterConsole(Gtk.ScrolledWindow):
     """An Interactive Console for GTK.
 
@@ -131,6 +155,7 @@ class GTKInterpreterConsole(Gtk.ScrolledWindow):
     def __init__(self, locals: Dict[str, object], banner=banner):
         Gtk.ScrolledWindow.__init__(self)
 
+        self.set_vexpand(True)
         self.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
 
         self.text = Gtk.TextView()
@@ -162,6 +187,7 @@ class GTKInterpreterConsole(Gtk.ScrolledWindow):
 
         self.stdout = TextViewWriter("stdout", self.text)
         self.stderr = TextViewWriter("stderr", self.text)
+        self.excepthook = ExceptionWriter()
 
         self.current_prompt = "ps1"
         locals["help"] = Help(self.stdout, locals)
@@ -202,7 +228,7 @@ class GTKInterpreterConsole(Gtk.ScrolledWindow):
         buffer = self.text.get_buffer()
         input_mark = buffer.get_mark("input")
         self.text.scroll_to_mark(input_mark, 0, True, 1, 1)
-        return False
+        return GLib.SOURCE_REMOVE
 
     def push(self, line):
         self.buffer.append(line)
@@ -211,7 +237,7 @@ class GTKInterpreterConsole(Gtk.ScrolledWindow):
 
         source = "\n".join(self.buffer)
 
-        with self.stdout, self.stderr:
+        with self.stdout, self.stderr, self.excepthook:
             more = self.interpreter.runsource(source, "<<console>>")
 
         if not more:
@@ -318,8 +344,9 @@ class GTKInterpreterConsole(Gtk.ScrolledWindow):
 
 
 def main(main_loop=True):
-    w = Gtk.ApplicationWindow()
-    w.set_default_size(640, 480)
+    window = Gtk.ApplicationWindow()
+
+    window.set_default_size(640, 480)
     console = GTKInterpreterConsole(locals())
 
     def key_event(widget, keyval, keycode, state):
@@ -327,22 +354,22 @@ def main(main_loop=True):
             app.quit()
         return False
 
-    w.connect("destroy", lambda w: app.quit())
     console.text_controller.connect("key-pressed", key_event)
 
     if Gtk.get_major_version() == 3:
-        w.add(console)
-        w.show_all()
+        window.connect("destroy", lambda w: app.quit())
+        window.add(console)
+        window.show_all()
     else:
-        w.set_child(console)
-        w.show()
+        window.set_child(console)
+        window.show()
 
     if main_loop:
 
         def on_activate(app):
-            app.add_window(w)
+            app.add_window(window)
 
-        app = Gtk.Application.new("org.gaphor.Console", 0)
+        app = Gtk.Application(application_id="org.gaphor.Console")
         app.connect("activate", on_activate)
         app.run()
 
