@@ -5,6 +5,9 @@ import os
 
 from gi.repository import Gdk, Gtk
 
+if Gtk.get_major_version() == 4:
+    from gi.repository import Adw
+
 from gaphor.abc import ActionProvider
 from gaphor.action import action
 from gaphor.i18n import gettext
@@ -16,7 +19,6 @@ log = logging.getLogger(__name__)
 
 
 class ConsoleWindow(UIComponent, ActionProvider):
-
     title = gettext("Gaphor Console")
     size = (400, 400)
 
@@ -45,7 +47,7 @@ class ConsoleWindow(UIComponent, ActionProvider):
         if not self.window:
             self.open()
         else:
-            self.window.set_property("has-focus", True)
+            self.window.present()
 
     def open(self):
         console = self.construct()
@@ -60,7 +62,7 @@ class ConsoleWindow(UIComponent, ActionProvider):
         window = (
             Gtk.Window.new(Gtk.WindowType.TOPLEVEL)
             if Gtk.get_major_version() == 3
-            else Gtk.Window.new()
+            else Adw.Window.new()
         )
         window.set_transient_for(self.main_window.window)
         window.set_title(self.title)
@@ -76,19 +78,22 @@ class ConsoleWindow(UIComponent, ActionProvider):
         if Gtk.get_major_version() == 3:
             console.show()
             window.add(console)
+            window.connect("destroy", self.close)
         else:
-            window.set_child(console)
+            box = Gtk.Box(orientation="vertical")
+            header_bar = Gtk.HeaderBar()
+            box.append(header_bar)
+            box.append(console)
+            window.set_content(box)
+            window.connect("close-request", self.close)
         window.show()
-
         self.window = window
 
         def key_event(widget, keyval, keycode, state):
             if keyval == Gdk.KEY_d and state & Gdk.ModifierType.CONTROL_MASK:
-                window.destroy()
+                self.close()
             return False
 
         console.text_controller.connect("key-pressed", key_event)
 
-        window.connect("destroy", self.close)
-
         return console
