@@ -4,10 +4,13 @@ system.
 These are things like preferences.
 """
 
+
 import ast
 import hashlib
+import logging
 import os
 import pprint
+from pathlib import Path
 from typing import Dict
 
 from gi.repository import GLib
@@ -17,6 +20,8 @@ from gaphor.core import event_handler
 from gaphor.core.modeling.event import ModelFlushed
 from gaphor.event import ModelLoaded, ModelSaved, SessionCreated
 
+log = logging.getLogger(__name__)
+
 
 def get_config_dir() -> str:
     """Return the directory where the user's config is stored.
@@ -42,7 +47,7 @@ def get_cache_dir() -> str:
     return cache_dir
 
 
-def file_hash(filename: str) -> str:
+def file_hash(filename) -> str:
     return hashlib.blake2b(str(filename).encode("utf-8"), digest_size=24).hexdigest()
 
 
@@ -124,10 +129,12 @@ class Properties(Service):
 
         if os.path.exists(filename) and os.path.isfile(filename):
 
-            with open(filename) as ifile:
-                data = ifile.read()
+            data = Path(filename).read_text()
+            try:
+                self._properties = ast.literal_eval(data)
+            except SyntaxError:
+                log.error("Invalid syntax in property file %s", filename)
 
-            self._properties = ast.literal_eval(data)
             for key, value in list(self._properties.items()):
                 self.event_manager.handle(PropertyChanged(key, None, value))
 
@@ -138,7 +145,7 @@ class Properties(Service):
         if not self.filename:
             return
 
-        with open(self.filename, "w") as ofile:
+        with open(self.filename, "w", encoding="utf-8") as ofile:
             pprint.pprint(self._properties, ofile)
 
     def get(self, key: str, default=_no_default):
@@ -151,9 +158,9 @@ class Properties(Service):
 
         try:
             return self._properties[key]
-        except KeyError:
+        except KeyError as e:
             if default is _no_default:
-                raise KeyError(f'No resource with name "{key}"')
+                raise KeyError(f'No resource with name "{key}"') from e
 
             self.set(key, default)
             return default
