@@ -11,7 +11,7 @@ NOTE: it would be nice to use actions in conjunction with functools.partial.
 """
 
 import logging
-from typing import Callable, List, Optional
+from typing import Callable, List
 
 from gaphor.abc import ActionProvider, Service
 from gaphor.action import action
@@ -26,7 +26,7 @@ from gaphor.core.modeling.event import (
     ElementCreated,
     ElementDeleted,
     ModelReady,
-    RevertibeEvent,
+    RevertibleEvent,
 )
 from gaphor.core.modeling.presentation import Presentation
 from gaphor.core.modeling.properties import association as association_property
@@ -76,7 +76,7 @@ class UndoManagerStateChanged(ServiceEvent):
 
 
 class NotInTransactionException(Exception):
-    """Raised when changes occur outside of a transaction."""
+    """Raised when changes occur outside a transaction."""
 
 
 class UndoManager(Service, ActionProvider):
@@ -99,10 +99,10 @@ class UndoManager(Service, ActionProvider):
         self._current_transaction = None
         self._undoing = 0
 
-        event_manager.subscribe(self.reset)
-        event_manager.subscribe(self.begin_transaction)
-        event_manager.subscribe(self.commit_transaction)
-        event_manager.subscribe(self.rollback_transaction)
+        event_manager.priority_subscribe(self.reset)
+        event_manager.priority_subscribe(self.begin_transaction)
+        event_manager.priority_subscribe(self.commit_transaction)
+        event_manager.priority_subscribe(self.rollback_transaction)
         self._register_undo_handlers()
         self._action_executed()
 
@@ -149,7 +149,9 @@ class UndoManager(Service, ActionProvider):
                 self._redo_stack = redo_stack
                 self._undo_stack = undo_stack
 
-            raise NotInTransactionException("Updating state outside of a transaction.")
+            raise NotInTransactionException(
+                f"Updating state outside of a transaction: {action.__doc__}."
+            )
 
     @event_handler(TransactionCommit)
     def commit_transaction(self, event=None):
@@ -189,7 +191,6 @@ class UndoManager(Service, ActionProvider):
         self._action_executed()
 
     def discard_transaction(self):
-
         self._current_transaction = None
 
         self._action_executed()
@@ -267,19 +268,16 @@ class UndoManager(Service, ActionProvider):
         self.event_manager.handle(UndoManagerStateChanged(self))
 
     def lookup(self, id: str) -> Element:
-        element: Optional[Element] = self.element_factory.lookup(id)
-        if not element:
+        if element := self.element_factory.lookup(id):
+            return element
+        else:
             raise ValueError(f"Element with id {id} not found in model")
-        return element
 
     #
     # Undo Handlers
     #
 
     def _register_undo_handlers(self):
-
-        logger.debug("Registering undo handlers")
-
         self.event_manager.subscribe(self.undo_reversible_event)
         self.event_manager.subscribe(self.undo_create_element_event)
         self.event_manager.subscribe(self.undo_delete_element_event)
@@ -289,9 +287,6 @@ class UndoManager(Service, ActionProvider):
         self.event_manager.subscribe(self.undo_association_delete_event)
 
     def _unregister_undo_handlers(self):
-
-        logger.debug("Unregistering undo handlers")
-
         self.event_manager.unsubscribe(self.undo_reversible_event)
         self.event_manager.unsubscribe(self.undo_create_element_event)
         self.event_manager.unsubscribe(self.undo_delete_element_event)
@@ -300,8 +295,8 @@ class UndoManager(Service, ActionProvider):
         self.event_manager.unsubscribe(self.undo_association_add_event)
         self.event_manager.unsubscribe(self.undo_association_delete_event)
 
-    @event_handler(RevertibeEvent)
-    def undo_reversible_event(self, event: RevertibeEvent):
+    @event_handler(RevertibleEvent)
+    def undo_reversible_event(self, event: RevertibleEvent):
         element_id = event.element.id
 
         def undo_reversible_event():
