@@ -8,7 +8,6 @@ __all__ = ["load", "save"]
 
 import io
 import logging
-import os.path
 from functools import partial
 
 from gaphor import application
@@ -16,7 +15,7 @@ from gaphor.core.modeling.collection import collection
 from gaphor.core.modeling.element import Element
 from gaphor.core.modeling.presentation import Presentation
 from gaphor.core.modeling.stylesheet import StyleSheet
-from gaphor.storage import parser
+from gaphor.storage.parser import GaphorLoader, parse_generator
 from gaphor.storage.xmlwriter import XMLWriter
 
 FILE_FORMAT_VERSION = "3.0"
@@ -82,7 +81,7 @@ def save_element(name, value, factory, writer):
     def save_reference(name, value):
         """Save a value as a reference to another element in the model.
 
-        This applies to both UML as well as canvas items.
+        This applies to both UML and canvas items.
         """
         if resolvable(value):
             writer.startElement(name, {})
@@ -179,9 +178,20 @@ def _load_elements_and_canvasitems(
         if version_lower_than(gaphor_version, (2, 9, 0)):
             elem = upgrade_flow_item_to_control_flow_item(elem, elements)
 
-        cls = modeling_language.lookup_element(elem.type)
-        assert cls, f"Type {elem.type} cannot be loaded: no such element"
+        if not (cls := modeling_language.lookup_element(elem.type)):
+            raise UnknownModelElementError(
+                f"Type {elem.type} cannot be loaded: no such element"
+            )
+
         if issubclass(cls, Presentation):
+            if "diagram" not in elem.references:
+                log.warning(
+                    "Removing element %s of type %s without diagram", elem.id, cls
+                )
+                assert not any(elem.id in e.references for e in elements.values())
+                del elements[elem.id]
+                return
+
             diagram_id = elem.references["diagram"]
             diagram_elem = elements[diagram_id]
             create_element(diagram_elem)
@@ -235,30 +245,24 @@ def load(filename, factory, modeling_language, status_queue=None):
             status_queue(status)
 
 
-def load_generator(filename, factory, modeling_language):
+def load_generator(file_obj, factory, modeling_language):
     """Load a file and create a model if possible.
 
     This function is a generator. It will yield values from 0 to 100 (%)
     to indicate its progression.
     """
-    if isinstance(filename, io.IOBase):
-        log.info("Loading file from file descriptor")
-    else:
-        log.info(f"Loading file {os.fsdecode(os.path.basename(filename))}")
-    try:
-        # Use the incremental parser and yield the percentage of the file.
-        loader = parser.GaphorLoader()
-        for percentage in parser.parse_generator(filename, loader):
-            if percentage:
-                yield percentage / 2
-            else:
-                yield percentage
-        elements = loader.elements
-        gaphor_version = loader.gaphor_version
+    assert isinstance(file_obj, io.IOBase)
 
-    except OSError:
-        log.exception("File could no be parsed")
-        raise
+    # Use the incremental parser and yield the percentage of the file.
+    loader = GaphorLoader()
+    for percentage in parse_generator(file_obj, loader):
+        if percentage:
+            yield percentage / 2
+        else:
+            yield percentage
+
+    elements = loader.elements
+    gaphor_version = loader.gaphor_version
 
     if version_lower_than(gaphor_version, (0, 17, 0)):
         raise ValueError(
@@ -269,18 +273,15 @@ def load_generator(filename, factory, modeling_language):
 
     factory.flush()
     with factory.block_events():
-        try:
-            for percentage in load_elements_generator(
-                elements, factory, modeling_language, gaphor_version
-            ):
-                if percentage:
-                    yield percentage / 2 + 50
-                else:
-                    yield percentage
-            yield 100
-        except Exception as e:
-            log.warning(f"file {filename} could not be loaded ({e})")
-            raise
+        for percentage in load_elements_generator(
+            elements, factory, modeling_language, gaphor_version
+        ):
+            if percentage:
+                yield percentage / 2 + 50
+            else:
+                yield percentage
+
+    yield 100
     factory.model_ready()
 
 
@@ -295,6 +296,10 @@ def version_lower_than(gaphor_version, version):
     return tuple(map(int, parts[:2])) < version[:2]
 
 
+class UnknownModelElementError(Exception):
+    pass
+
+
 # since 2.2.0
 def upgrade_ensure_style_sheet_is_present(factory):
     style_sheet = next(factory.select(StyleSheet), None)
