@@ -4,7 +4,6 @@ and diagram windows."""
 import logging
 import os
 import sys
-from pathlib import Path
 from typing import Optional
 
 import darkdetect
@@ -12,13 +11,15 @@ import gi
 
 if os.getenv("GAPHOR_USE_GTK") != "NONE":
     # Allow to explicitly *not* initialize GTK (for docs, mainly)
-    gtk_version = "4.0" if os.getenv("GAPHOR_USE_GTK") == "4" else "3.0"
-    gtk_source_version = "5" if os.getenv("GAPHOR_USE_GTK") == "4" else "4"
+    gtk_version = "3.0" if os.getenv("GAPHOR_USE_GTK") == "3" else "4.0"
+    gtk_source_version = "4" if os.getenv("GAPHOR_USE_GTK") == "3" else "5"
 
     gi.require_version("Gtk", gtk_version)
     gi.require_version("Gdk", gtk_version)
-    if gtk_version == "3.0":
-        gi.require_version("GtkSource", gtk_source_version)
+    gi.require_version("GtkSource", gtk_source_version)
+    if gtk_version == "4.0":
+        gi.require_version("Adw", "1")
+        from gi.repository import Adw
 
 
 from gi.repository import Gdk, Gio, GLib, Gtk
@@ -30,11 +31,10 @@ from gaphor.ui.actiongroup import apply_application_actions
 from gaphor.ui.macosshim import macos_init
 
 APPLICATION_ID = "org.gaphor.Gaphor"
-HOME = str(Path.home())
 LOG_FORMAT = "%(name)s %(levelname)s %(message)s"
 
 
-def main(argv=sys.argv):
+def main(argv=sys.argv) -> int:
     """Start Gaphor from the command line.  This function creates an option
     parser for retrieving arguments and options from the command line.  This
     includes a Gaphor model to load.
@@ -49,7 +49,7 @@ def main(argv=sys.argv):
 
     if has_option("-v", "--version"):
         print(f"Gaphor {distribution().version}")
-        return
+        return 0
 
     if has_option("-d", "--debug"):
         logging.basicConfig(level=logging.INFO, format=LOG_FORMAT)
@@ -61,9 +61,16 @@ def main(argv=sys.argv):
 
     # Set dark mode for non-FreeDesktop platforms only:
     if sys.platform in ("darwin", "win32"):
-        Gtk.Settings.get_default().set_property(
-            "gtk-application-prefer-dark-theme", darkdetect.isDark()
-        )
+        if Gtk.get_major_version() == 3:
+            Gtk.Settings.get_default().set_property(
+                "gtk-application-prefer-dark-theme", darkdetect.isDark()
+            )
+        else:
+            Adw.StyleManager.get_default().set_color_scheme(
+                Adw.ColorScheme.PREFER_DARK
+                if darkdetect.isDark()
+                else Adw.ColorScheme.PREFER_LIGHT
+            )
 
     if has_option("-p", "--profiler"):
 
@@ -71,16 +78,16 @@ def main(argv=sys.argv):
         import pstats
 
         with cProfile.Profile() as profile:
-            profile.runcall(run, argv)
+            exit_code = profile.runcall(run, argv)
 
         profile_stats = pstats.Stats(profile)
         profile_stats.strip_dirs().sort_stats("time").print_stats(50)
+        return exit_code
 
-    else:
-        run(argv)
+    return run(argv)
 
 
-def run(args):
+def run(args: list[str]) -> int:
     application: Optional[Application] = None
 
     def app_startup(gtk_app):
@@ -105,15 +112,17 @@ def run(args):
             gtk_app.quit()
 
         try:
-            application = Application()
+            application = Application(gtk_app=gtk_app)
             apply_application_actions(application, gtk_app)
-            if macos_init:
-                macos_init(application)
+            macos_init(application)
             event_manager = application.get_service("event_manager")
             event_manager.subscribe(on_session_created)
             event_manager.subscribe(on_quit)
-            application.get_service("greeter").init(gtk_app)
+            application.get_service(
+                "self_test" if "--self-test" in args else "greeter"
+            ).init(gtk_app)
         except Exception:
+            gtk_app.exit_code = 1
             gtk_app.quit()
             raise
 
@@ -131,14 +140,21 @@ def run(args):
             for file in files:
                 application.new_session(filename=file.get_path())
 
-    gtk_app = Gtk.Application(
-        application_id=APPLICATION_ID, flags=Gio.ApplicationFlags.HANDLES_OPEN
-    )
+    if gtk_version == "3.0":
+        gtk_app = Gtk.Application(
+            application_id=APPLICATION_ID, flags=Gio.ApplicationFlags.HANDLES_OPEN
+        )
+    else:
+        gtk_app = Adw.Application(
+            application_id=APPLICATION_ID, flags=Gio.ApplicationFlags.HANDLES_OPEN
+        )
+    gtk_app.exit_code = 0
     add_main_options(gtk_app)
     gtk_app.connect("startup", app_startup)
     gtk_app.connect("activate", app_activate)
     gtk_app.connect("open", app_open)
     gtk_app.run(args)
+    return gtk_app.exit_code
 
 
 def add_main_options(gtk_app):
@@ -178,3 +194,11 @@ def add_main_options(gtk_app):
         "Run in profiler",
         None,
     )
+    gtk_app.add_main_option(
+        "self-test",
+        0,
+        GLib.OptionFlags.NONE,
+        GLib.OptionArg.NONE,
+        "Run self test and exit",
+        None,
+    )
