@@ -27,18 +27,19 @@ from gaphor.diagram.tools.magnet import MagnetPainter
 from gaphor.diagram.tools.placement import create_item, open_editor
 from gaphor.event import Notification
 from gaphor.transaction import Transaction
-from gaphor.ui.event import DiagramSelectionChanged, ToolSelected
+from gaphor.ui.event import DiagramClosed, DiagramSelectionChanged, ToolSelected
 
 log = logging.getLogger(__name__)
 
 
 @functools.lru_cache(maxsize=1)
 def placement_icon_base():
-    with importlib.resources.path("gaphor.ui", "placement-icon-base.png") as f:
-        return GdkPixbuf.Pixbuf.new_from_file_at_scale(str(f), 64, 64, True)
+    f = importlib.resources.files("gaphor") / "ui" / "placement-icon-base.png"
+    return GdkPixbuf.Pixbuf.new_from_file_at_scale(str(f), 64, 64, True)
 
 
-GtkView.set_css_name("diagramview")
+if hasattr(GtkView, "set_css_name"):
+    GtkView.set_css_name("diagramview")
 
 
 if Gtk.get_major_version() == 3:
@@ -96,7 +97,6 @@ else:
 
 
 class DiagramPage:
-
     if Gtk.get_major_version() == 3:
         VIEW_TARGET_STRING = 0
         VIEW_TARGET_ELEMENT_ID = 1
@@ -142,6 +142,7 @@ class DiagramPage:
 
         view = GtkView(selection=Selection())
         if Gtk.get_major_version() == 3:
+            view.add_events(Gdk.EventMask.SMOOTH_SCROLL_MASK)
             view.drag_dest_set(
                 Gtk.DestDefaults.ALL,
                 DiagramPage.VIEW_DND_TARGETS,
@@ -219,7 +220,7 @@ class DiagramPage:
     @event_handler(ElementDeleted)
     def _on_element_delete(self, event: ElementDeleted):
         if event.element is self.diagram:
-            self.close()
+            self.event_manager.handle(DiagramClosed(self.diagram))
 
     @event_handler(AttributeUpdated)
     def _on_attribute_updated(self, event: AttributeUpdated):
@@ -240,8 +241,6 @@ class DiagramPage:
         assert self.widget
         if Gtk.get_major_version() == 3:
             self.widget.destroy()
-        elif parent := self.widget.get_parent():
-            parent.remove(self.widget)
 
         self.event_manager.unsubscribe(self._on_element_delete)
         self.event_manager.unsubscribe(self._on_attribute_updated)
@@ -303,7 +302,8 @@ class DiagramPage:
 
     def _on_view_selection_changed(self, item):
         view = self.view
-        assert view
+        if not view:
+            return
         selection = view.selection
         self.event_manager.handle(
             DiagramSelectionChanged(
