@@ -1,9 +1,10 @@
 """The element editor is a utility window used for editing elements."""
 
+import importlib.resources
 import logging
 from typing import Optional
 
-from gi.repository import GLib, Gtk
+from gi.repository import GLib, Gtk, GtkSource
 
 from gaphor.abc import ActionProvider
 from gaphor.core import Transaction, action, event_handler
@@ -18,10 +19,6 @@ from gaphor.diagram.propertypages import PropertyPages, new_resource_builder
 from gaphor.ui.abc import UIComponent
 from gaphor.ui.event import DiagramSelectionChanged
 
-if Gtk.get_major_version() == 3:
-    from gi.repository import GtkSource
-
-
 log = logging.getLogger(__name__)
 
 
@@ -90,7 +87,7 @@ class ElementEditor(UIComponent, ActionProvider):
 
     @action(
         name="show-editors",
-        shortcut="<Primary>e",
+        shortcut="F9",
         state=lambda self: self.properties.get("show-editors", True),  # type: ignore[no-any-return]
     )
     def toggle_editor_visibility(self, active):
@@ -264,8 +261,11 @@ class PreferencesStack:
     def __init__(self, event_manager, element_factory):
         self.event_manager = event_manager
         self.element_factory = element_factory
-        if Gtk.get_major_version() == 3:
-            self.lang_manager = GtkSource.LanguageManager.get_default()
+        self.lang_manager = GtkSource.LanguageManager.get_default()
+        if Gtk.get_major_version() != 3:
+            self.lang_manager.append_search_path(
+                str(importlib.resources.files("gaphor") / "ui" / "language-specs")
+            )
 
         def tx_update_style_sheet(style_sheet, text):
             self._in_update = 1
@@ -280,8 +280,11 @@ class PreferencesStack:
         self.style_sheet_buffer = builder.get_object("style-sheet-buffer")
         self.style_sheet_view = builder.get_object("style-sheet-view")
 
-        if Gtk.get_major_version() == 3:
-            self.style_sheet_buffer.set_language(self.lang_manager.get_language("css"))
+        self.style_sheet_buffer.set_language(
+            self.lang_manager.get_language(
+                "css" if Gtk.get_major_version() == 3 else "gaphorcss"
+            )
+        )
 
         self.event_manager.subscribe(self._model_ready)
         self.event_manager.subscribe(self._style_sheet_created)
