@@ -1,10 +1,16 @@
 """The file service is responsible for loading and saving the user data."""
 
+from __future__ import annotations
+
 import logging
+from pathlib import Path
 from queue import Queue
 
 from gaphas.decorators import g_async
-from gi.repository import GLib, Gtk
+from gi.repository import Gtk
+
+if Gtk.get_major_version() == 4:
+    from gi.repository import Adw
 
 from gaphor import UML
 from gaphor.abc import ActionProvider, Service
@@ -18,6 +24,7 @@ from gaphor.event import (
     SessionShutdownRequested,
 )
 from gaphor.storage import storage
+from gaphor.storage.parser import MergeConflictDetected
 from gaphor.ui.errorhandler import error_handler
 from gaphor.ui.filedialog import GAPHOR_FILTER, save_file_dialog
 from gaphor.ui.statuswindow import StatusWindow
@@ -97,7 +104,7 @@ class FileManager(Service, ActionProvider):
         """
 
         if filename != self._filename:
-            self._filename = filename
+            self._filename = Path(filename) if filename else None
 
     def load(self, filename):
         """Load the Gaphor model from the supplied file name.
@@ -108,40 +115,61 @@ class FileManager(Service, ActionProvider):
         successful, the filename is set.
         """
         # First claim file name, so any other files will be opened in a different session
-        self.filename = filename
+        self.filename = Path(filename)
         queue: Queue[int] = Queue(0)
-        status_window = StatusWindow(
-            gettext("Loading…"),
-            gettext("Loading model from {filename}").format(filename=filename),
-            parent=self.main_window.window,
-            queue=queue,
-        )
+        status_window = None
 
-        self._load(filename, ModelLoaded(self, filename), queue, status_window)
+        @g_async()
+        def create_status_window():
+            nonlocal status_window
+            status_window = StatusWindow(
+                gettext("Loading…"),
+                gettext("Loading model from {filename}").format(filename=filename),
+                parent=self.parent_window,
+                queue=queue,
+            )
+
+        def done():
+            if status_window:
+                status_window.destroy()
+
+        create_status_window()
+        self._load_async(filename, queue, done)
 
     def load_template(self, template):
-        self._load(template, ModelLoaded(self))
+        storage.load(template, self.element_factory, self.modeling_language)
+        self.event_manager.handle(ModelLoaded(self))
+
+    def _load_async(self, filename: Path, queue=None, done=None):
+        assert isinstance(filename, Path)
 
-    def _load(self, filename, model_loaded_event, queue=None, status_window=None):
         # Use low prio, so screen updates do happen
-        @g_async(priority=GLib.PRIORITY_DEFAULT_IDLE)
+        @g_async()
         def async_loader():
             try:
-                for percentage in storage.load_generator(
-                    filename,
-                    self.element_factory,
-                    self.modeling_language,
-                ):
-                    if queue:
-                        queue.put(percentage)
-                    yield percentage
+                try:
+                    yield from open_model(encoding="utf-8")
+                except UnicodeDecodeError:
+                    # try to load without encoding, for older models saved on windows
+                    yield from open_model(encoding=None)
 
-                self.event_manager.handle(model_loaded_event)
+                self.event_manager.handle(ModelLoaded(self, filename))
+            except MergeConflictDetected:
+                self.filename = None
+                error_handler(
+                    message=gettext("Merge conflict in model “{filename}”.").format(
+                        filename=filename
+                    ),
+                    secondary_message=gettext(
+                        "It looks like this model file contains a merge conflict."
+                    ),
+                    window=self.parent_window,
+                    close=lambda: self.event_manager.handle(SessionShutdown(self)),
+                )
+                # For now load the default model until we allow users to resolve the merge conflict.
+                load_default_model(self.element_factory)
             except Exception:
                 self.filename = None
-                if status_window:
-                    status_window.destroy()
-                log.exception(f"Unable to open model “{filename}”.", stack_info=True)
                 error_handler(
                     message=gettext("Unable to open model “{filename}”.").format(
                         filename=filename
@@ -149,18 +177,28 @@ class FileManager(Service, ActionProvider):
                     secondary_message=gettext(
                         "This file does not contain a valid Gaphor model."
                     ),
-                    window=self.main_window.window,
+                    window=self.parent_window,
+                    close=lambda: self.event_manager.handle(SessionShutdown(self)),
                 )
-                load_default_model(self.element_factory)
-                raise
             finally:
-                if status_window:
-                    status_window.destroy()
+                if done:
+                    done()
+
+        def open_model(encoding):
+            with open(filename, encoding=encoding) as file_obj:
+                for percentage in storage.load_generator(
+                    file_obj,
+                    self.element_factory,
+                    self.modeling_language,
+                ):
+                    if queue:
+                        queue.put(percentage)
+                    yield percentage
 
         for _ in async_loader():
             pass
 
-    def save(self, filename):
+    def save(self, filename, on_save_done=None):
         """Save the current UML model to the specified file name.
 
         Before writing the model file, this will verify that there are
@@ -169,22 +207,21 @@ class FileManager(Service, ActionProvider):
         GIdleThread is executed.  This thread actually saves the model.
         """
 
-        if not (filename and len(filename)):
+        if not filename or (filename.exists() and not filename.is_file()):
             return
 
-        main_window = self.main_window
-        queue: Queue[int] = Queue(0)
+        queue: Queue[int] = Queue()
         status_window = StatusWindow(
             gettext("Saving…"),
             gettext("Saving model to {filename}").format(filename=filename),
-            parent=main_window.window if main_window else None,
+            parent=self.parent_window,
             queue=queue,
         )
 
-        @g_async(priority=GLib.PRIORITY_DEFAULT_IDLE)
+        @g_async()
         def async_saver():
             try:
-                with open(filename, "w") as out:
+                with filename.open("w", encoding="utf-8") as out:
                     for percentage in storage.save_generator(out, self.element_factory):
                         queue.put(percentage)
                         yield
@@ -197,15 +234,21 @@ class FileManager(Service, ActionProvider):
                         filename=filename
                     ),
                     secondary_message=error_message(e),
-                    window=main_window.window if main_window else None,
+                    window=self.parent_window,
                 )
                 raise
             finally:
                 status_window.destroy()
+            if on_save_done:
+                on_save_done()
 
         for _ in async_saver():
             pass
 
+    @property
+    def parent_window(self):
+        return self.main_window.window if self.main_window else None
+
     @action(name="file-save", shortcut="<Primary>s")
     def action_save(self):
         """Save the file. Depending on if there is a file name, either perform
@@ -214,13 +257,10 @@ class FileManager(Service, ActionProvider):
         Returns True if the saving actually succeeded.
         """
 
-        filename = self.filename
-
-        if not filename:
-            return self.action_save_as()
-
-        self.save(filename)
-        return True
+        if filename := self.filename:
+            self.save(filename)
+        else:
+            self.action_save_as()
 
     @action(name="file-save-as", shortcut="<Primary><Shift>s")
     def action_save_as(self):
@@ -230,17 +270,14 @@ class FileManager(Service, ActionProvider):
         Returns True if the saving actually happened.
         """
 
-        if filename := save_file_dialog(
+        save_file_dialog(
             gettext("Save Gaphor Model As"),
-            parent=self.main_window.window,
+            self.save,
+            parent=self.parent_window,
             filename=self.filename,
             extension=".gaphor",
             filters=GAPHOR_FILTER,
-        ):
-            self.save(filename)
-            return True
-
-        return False
+        )
 
     @event_handler(SessionCreated)
     def _on_session_created(self, event: SessionCreated) -> None:
@@ -262,39 +299,72 @@ class FileManager(Service, ActionProvider):
         def confirm_shutdown():
             self.event_manager.handle(SessionShutdown(self))
 
-        if self.main_window.model_changed:
-            response = save_changes_before_closing_dialog(self.main_window.window)
-            if response == Gtk.ResponseType.YES:
-                saved = self.action_save()
-                if saved:
-                    confirm_shutdown()
-            if response == Gtk.ResponseType.REJECT:
+        def response(answer):
+            # Gtk.ResponseType.YES is GTK3, save is GTK4
+            if answer in [Gtk.ResponseType.YES, "save"]:
+                if filename := self.filename:
+                    self.save(filename, on_save_done=confirm_shutdown)
+
+                else:
+                    save_file_dialog(
+                        gettext("Save Gaphor Model As"),
+                        lambda filename: self.save(
+                            filename, on_save_done=confirm_shutdown
+                        ),
+                        parent=self.parent_window,
+                        filename=self.filename,
+                        extension=".gaphor",
+                        filters=GAPHOR_FILTER,
+                    )
+            # Gtk.ResponseType.REJECT is GTK3, discard is GTK4
+            elif answer in [Gtk.ResponseType.REJECT, "discard"]:
                 confirm_shutdown()
+
+        if self.main_window.model_changed:
+            save_changes_before_closing_dialog(self.parent_window, response)
         else:
             confirm_shutdown()
 
 
-def save_changes_before_closing_dialog(window: Gtk.Window) -> Gtk.ResponseType:
-    dialog = Gtk.MessageDialog(
-        window,
-        Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
-        Gtk.MessageType.WARNING,
-    )
-    dialog.props.text = gettext("Save changes before closing?")
-    dialog.props.secondary_text = gettext(
-        "Closing will cause any unsaved changes to be discarded."
+def save_changes_before_closing_dialog(window: Gtk.Window, handler) -> None:
+    title = gettext("Save Changes?")
+    body = gettext(
+        "The open model contains unsaved changes. Changes which are not saved will be permanently lost."
     )
+    if Gtk.get_major_version() == 3:
+        dialog = Gtk.MessageDialog(
+            message_type=Gtk.MessageType.WARNING,
+            text=title,
+            secondary_text=body,
+        )
+        dialog.set_transient_for(window)
+        dialog.set_modal(True)
+        dialog.add_buttons(
+            gettext("Cancel"),
+            Gtk.ResponseType.CANCEL,
+            gettext("Discard"),
+            Gtk.ResponseType.REJECT,
+            gettext("Save"),
+            Gtk.ResponseType.YES,
+        )
+        dialog.set_default_response(Gtk.ResponseType.YES)
+    else:
+        dialog = Adw.MessageDialog.new(
+            window,
+            title,
+        )
+        dialog.set_body(body)
+        dialog.add_response("cancel", gettext("Cancel"))
+        dialog.add_response("discard", gettext("Discard"))
+        dialog.add_response("save", gettext("Save"))
+        dialog.set_response_appearance("discard", Adw.ResponseAppearance.DESTRUCTIVE)
+        dialog.set_response_appearance("save", Adw.ResponseAppearance.SUGGESTED)
+        dialog.set_default_response("save")
+        dialog.set_close_response("cancel")
 
-    dialog.add_buttons(
-        gettext("Close _without saving changes"),
-        Gtk.ResponseType.REJECT,
-        Gtk.STOCK_CANCEL,
-        Gtk.ResponseType.CANCEL,
-        Gtk.STOCK_SAVE,
-        Gtk.ResponseType.YES,
-    )
-    dialog.set_default_response(Gtk.ResponseType.YES)
-    response = dialog.run()
-    dialog.destroy()
+    def response(dialog, answer):
+        dialog.destroy()
+        handler(answer)
 
-    return response
+    dialog.connect("response", response)
+    dialog.show()
