@@ -1,5 +1,7 @@
 """The main application window."""
 
+from __future__ import annotations
+
 import importlib.resources
 import logging
 from pathlib import Path
@@ -19,7 +21,6 @@ from gaphor.event import (
 from gaphor.i18n import translated_ui_string
 from gaphor.services.modelinglanguage import ModelingLanguageChanged
 from gaphor.services.undomanager import UndoManagerStateChanged
-from gaphor.ui import HOME
 from gaphor.ui.abc import UIComponent
 from gaphor.ui.actiongroup import window_action_group
 from gaphor.ui.layout import deserialize, is_maximized
@@ -39,9 +40,8 @@ def create_hamburger_model(export_menu, tools_menu):
     model = Gio.Menu.new()
 
     part = Gio.Menu.new()
-    part.append(gettext("Open a Model…"), "app.file-open")
-    part.append(gettext("Recent Models…"), "app.recent-files")
     part.append(gettext("New Model…"), "app.new-model")
+    part.append(gettext("Open Model…"), "app.file-open")
     model.append_section(None, part)
 
     part = Gio.Menu.new()
@@ -55,8 +55,8 @@ def create_hamburger_model(export_menu, tools_menu):
     model.append_section(None, part)
 
     part = Gio.Menu.new()
-    part.append(gettext("Keyboard Shortcuts"), "win.shortcuts")
-    part.append(gettext("About Gaphor"), "win.about")
+    part.append(gettext("Keyboard Shortcuts"), "app.shortcuts")
+    part.append(gettext("About Gaphor"), "app.about")
     model.append_section(None, part)
 
     return model
@@ -122,11 +122,12 @@ class MainWindow(Service, ActionProvider):
         self.tools_menu = tools_menu
 
         self.window: Gtk.Window = None
+        self.action_group: Gio.ActionGroup = None
         self.title: Gtk.Label = None
+        self.modified: Gtk.Label = None
         self.subtitle: Gtk.Label = None
-        self.filename = None
+        self.filename: Path | None = None
         self.model_changed = False
-        self.layout = None
         self.modeling_language_name = None
         self.diagram_types = None
         self.in_app_notifier = None
@@ -176,6 +177,8 @@ class MainWindow(Service, ActionProvider):
         )
 
         self.title = builder.get_object("title")
+        if Gtk.get_major_version() != 3:
+            self.modified = builder.get_object("modified")
         self.subtitle = builder.get_object("subtitle")
         self.set_title()
 
@@ -185,28 +188,31 @@ class MainWindow(Service, ActionProvider):
             comp = self.get_ui_component(name)
             return comp.open()
 
-        with importlib.resources.open_text("gaphor.ui", "layout.xml") as f:
-            main_content = builder.get_object("main-content")
-            self.layout = deserialize(main_content, f.read(), _factory, self.properties)
+        main_content = builder.get_object("main-content")
+        deserialize(
+            main_content,
+            (importlib.resources.files("gaphor.ui") / "layout.xml").read_text(),
+            _factory,
+            self.properties,
+        )
 
-        action_group, shortcuts = window_action_group(self.component_registry)
-        self.window.insert_action_group("win", action_group)
-
-        if Gtk.get_major_version() == 3:
-            self.window.add_accel_group(shortcuts)
-        else:
-            self.window.add_controller(Gtk.ShortcutController.new_for_model(shortcuts))
+        self.action_group, shortcuts = window_action_group(self.component_registry)
+        self.window.insert_action_group("win", self.action_group)
 
         self._on_modeling_language_selection_changed()
 
         self.window.set_resizable(True)
         if Gtk.get_major_version() == 3:
             self.window.show_all()
-            self.window.connect("delete-event", self._on_window_delete)
+            self.window.add_accel_group(shortcuts)
+            self.window.connect("delete-event", self._on_window_close_request)
             self.window.connect("size-allocate", self._on_window_size_allocate)
         else:
+            self.window.add_controller(Gtk.ShortcutController.new_for_model(shortcuts))
+            self.window.connect("close-request", self._on_window_close_request)
+            self.window.connect("notify::default-height", self._on_window_size_changed)
+            self.window.connect("notify::default-width", self._on_window_size_changed)
             self.window.show()
-            # TODO: GTK4 - handle window delete and size allocation
 
         self.window.connect("notify::is-active", self._on_window_active)
 
@@ -223,14 +229,19 @@ class MainWindow(Service, ActionProvider):
             return
 
         if self.filename:
-            p = Path(self.filename)
+            p = self.filename
             title = p.stem
-            subtitle = str(p).replace(HOME, "~")
+            subtitle = str(p).replace(str(Path.home()), "~")
         else:
             title = "Gaphor"
             subtitle = gettext("New model")
-        if self.model_changed:
-            title += " [" + gettext("edited") + "]"
+
+        if Gtk.get_major_version() == 3:
+            if self.model_changed:
+                title += " [" + gettext("edited") + "]"
+        else:
+            self.modified.set_visible(self.model_changed)
+
         self.title.set_text(title)
         self.subtitle.set_text(subtitle)
         self.window.set_title(title)
@@ -238,9 +249,11 @@ class MainWindow(Service, ActionProvider):
     # Signal callbacks:
 
     @event_handler(SessionCreated, ModelLoaded, ModelSaved)
-    def _on_file_manager_state_changed(self, event):
+    def _on_file_manager_state_changed(
+        self, event: SessionCreated | ModelLoaded | ModelSaved
+    ):
         self.model_changed = False
-        self.filename = event.filename
+        self.filename = Path(event.filename) if event.filename else None
         self.set_title()
         if self.window:
             self.window.present()
@@ -254,19 +267,15 @@ class MainWindow(Service, ActionProvider):
 
     @event_handler(ActionEnabled)
     def _on_action_enabled(self, event):
-        if Gtk.get_major_version() == 3:
-            ag = self.window.get_action_group(event.scope)
-            a = ag.lookup_action(event.name)
+        if self.action_group and event.scope == "win":
+            a = self.action_group.lookup_action(event.name)
             a.set_enabled(event.enabled)
-        else:
-            # TODO: GTK4 - enable an action (shortcut?)
-            ...
 
     @event_handler(ModelingLanguageChanged)
     def _on_modeling_language_selection_changed(self, event=None):
         if self.modeling_language_name:
             self.modeling_language_name.set_label(
-                gettext("Profile: {}").format(self.modeling_language.name)
+                gettext("Profile: {name}").format(name=self.modeling_language.name)
             )
         if self.diagram_types:
             popup_set_model(
@@ -276,12 +285,20 @@ class MainWindow(Service, ActionProvider):
     def _on_window_active(self, window, prop):
         self.event_manager.handle(ActiveSessionChanged(self))
 
-    def _on_window_delete(self, window, event):
+    def _on_window_close_request(self, window, event=None):
         self.event_manager.handle(SessionShutdownRequested(self))
         return True
 
-    def _on_window_size_allocate(self, window, allocation):
-        """Store the window size in a property."""
-        if not is_maximized(window):
-            width, height = window.get_size()
-            self.properties.set("ui.window-size", (width, height))
+    if Gtk.get_major_version() == 3:
+
+        def _on_window_size_allocate(self, window, allocation):
+            if not is_maximized(window):
+                width, height = window.get_size()
+                self.properties.set("ui.window-size", (width, height))
+
+    else:
+
+        def _on_window_size_changed(self, window, gspec):
+            if not is_maximized(window):
+                width, height = window.get_default_size()
+                self.properties.set("ui.window-size", (width, height))
