@@ -20,9 +20,9 @@ class MenuFragment(Service):
     def menu(self):
         return self._menu
 
-    def add_actions(self, action_provider):
+    def add_actions(self, action_provider, scope="win"):
         section = Gio.Menu.new()
-        for method_name, action in iter_actions(action_provider, "win"):
+        for method_name, action in iter_actions(action_provider, scope):
             section.append(action.label, f"{action.scope}.{action.name}")
         if section.get_n_items():
             self._menu.append_section(None, section)
