@@ -21,7 +21,7 @@ from gaphor.diagram.deletable import deletable
 from gaphor.transaction import Transaction
 from gaphor.ui.abc import UIComponent
 from gaphor.ui.actiongroup import create_action_group
-from gaphor.ui.event import DiagramOpened, DiagramSelectionChanged
+from gaphor.ui.event import DiagramOpened, DiagramSelectionChanged, ElementOpened
 from gaphor.ui.mainwindow import create_diagram_types_model
 from gaphor.ui.namespacemodel import (
     RELATIONSHIPS,
@@ -43,7 +43,11 @@ def popup_model(element, modeling_language):
     model = Gio.Menu.new()
 
     part = Gio.Menu.new()
-    part.append(gettext("_Open"), "tree-view.open")
+
+    part.append(
+        gettext("_Open") if isinstance(element, Diagram) else gettext("Add to diagram"),
+        "tree-view.open",
+    )
     part.append(gettext("_Rename"), "tree-view.rename")
     model.append_section(None, part)
 
@@ -184,18 +188,26 @@ class Namespace(UIComponent, ActionProvider):
             self.select_element(focused_item.subject)
 
     def _on_show_popup(self, ctrl, n_press, x, y):
+        selected_element = self.get_selected_element()
+        if selected_element in (None, RELATIONSHIPS):
+            return
+
         if Gtk.get_major_version() == 3:
             menu = Gtk.Menu.new_from_model(
-                popup_model(self.get_selected_element(), self.modeling_language)
+                popup_model(selected_element, self.modeling_language)
             )
             menu.attach_to_widget(self.view, None)
             menu.popup_at_pointer(None)
         else:
             menu = Gtk.PopoverMenu.new_from_model(
-                popup_model(self.get_selected_element(), self.modeling_language)
+                popup_model(selected_element, self.modeling_language)
             )
-            menu.set_pointing_to(Gdk.Rectangle(x, y, 1, 1))
-            menu.set_offset(x, y)
+            gdk_rect = Gdk.Rectangle()
+            gdk_rect.x = x
+            gdk_rect.y = y
+            gdk_rect.width = gdk_rect.height = 1
+
+            menu.set_pointing_to(gdk_rect)
             menu.set_has_arrow(False)
             menu.set_parent(self.view)
             menu.popup()
@@ -207,7 +219,7 @@ class Namespace(UIComponent, ActionProvider):
         return False
 
     def _on_view_row_activated(self, view, path, column):
-        """Double click on an element in the tree view."""
+        """Double-click on an element in the tree view."""
         if Gtk.get_major_version() == 3:
             view.get_action_group("tree-view").lookup_action("open").activate()
         else:
@@ -219,7 +231,6 @@ class Namespace(UIComponent, ActionProvider):
         if Gtk.get_major_version() == 3:
             action_group = view.get_action_group("tree-view")
 
-            action_group.lookup_action("open").set_enabled(isinstance(element, Diagram))
             action_group.lookup_action("create-package").set_enabled(
                 isinstance(element, UML.Package)
             )
@@ -229,9 +240,6 @@ class Namespace(UIComponent, ActionProvider):
             action_group.lookup_action("rename").set_enabled(
                 isinstance(element, (Diagram, UML.NamedElement))
             )
-        else:
-            # TODO: GTK4 - enable/disable actions based on view state
-            ...
 
     def _on_view_destroyed(self, widget):
         self.close()
@@ -278,7 +286,7 @@ class Namespace(UIComponent, ActionProvider):
         if isinstance(element, Diagram):
             self.event_manager.handle(DiagramOpened(element))
         else:
-            log.debug(f"No action defined for element {type(element).__name__}")
+            self.event_manager.handle(ElementOpened(element))
 
     @action(name="tree-view.show-in-diagram")
     def tree_view_show_in_diagram(self, diagram_id: str) -> None:
