@@ -1,5 +1,7 @@
 """Defines a status window class for displaying the progress of a queue."""
 
+
+import contextlib
 from queue import Empty
 
 from gi.repository import Gdk, GLib, Gtk, Pango
@@ -116,15 +118,12 @@ def progress_idle_handler(progress_bar, queue):
     The percentage is retrieved from the queue until it is empty.  The
     progress bar is then updated with the current percentage.
     """
-
     percentage = 0
-    try:
+    with contextlib.suppress(Empty):
         percentage = queue.get(block=False)
-    except Empty:
-        pass
     if percentage:
         progress_bar.set_fraction(min(percentage, 100.0) / 100.0)
-    return True
+    return GLib.SOURCE_CONTINUE
 
 
 def remove_idle_handler(window, idle_id):
