@@ -5,15 +5,16 @@ This is the toolbox in the lower left of the screen.
 
 import functools
 import logging
+import sys
 from typing import Optional, Tuple
 
-from gi.repository import Gdk, GObject, Gtk
+from gi.repository import Gdk, GLib, GObject, Gtk
 
 from gaphor.core.eventmanager import EventManager, event_handler
 from gaphor.diagram.diagramtoolbox import ToolboxDefinition
 from gaphor.diagram.event import ToolCompleted
 from gaphor.diagram.hoversupport import flowbox_add_hover_support
-from gaphor.event import TransactionCommit
+from gaphor.diagram.tools.dnd import ToolboxActionDragData
 from gaphor.services.modelinglanguage import (
     ModelingLanguageChanged,
     ModelingLanguageService,
@@ -26,7 +27,6 @@ log = logging.getLogger(__name__)
 
 
 class Toolbox(UIComponent):
-
     if Gtk.get_major_version() == 3:
         TARGET_STRING = 0
         TARGET_TOOLBOX_ACTION = 1
@@ -93,20 +93,27 @@ class Toolbox(UIComponent):
 
     def create_toolbox(self, toolbox_actions: ToolboxDefinition) -> Gtk.Box:
         toolbox = Gtk.Box.new(Gtk.Orientation.VERTICAL, 0)
-        toolbox.set_name("toolbox")
         expanded = self.expanded_sections()
 
-        def on_expanded(widget, _prop, index):
-            self.expanded_sections(index, widget.get_property("expanded"))
+        def on_expanded(widget, _prop, index, revealer):
+            expanded = widget.get_property("expanded")
+            self.expanded_sections(index, expanded)
+            revealer.set_reveal_child(expanded)
 
         for index, (title, items) in enumerate(toolbox_actions):
+            revealer = Gtk.Revealer.new()
+            revealer.set_transition_type(Gtk.RevealerTransitionType.SLIDE_UP)
+            revealer.set_reveal_child(expanded[index])
             expander = Gtk.Expander.new(title)
             expander.set_property("expanded", expanded[index])
-            expander.connect("notify::expanded", on_expanded, index)
+            expander.connect("notify::expanded", on_expanded, index, revealer)
             flowbox = Gtk.FlowBox.new()
             flowbox.set_homogeneous(True)
+            flowbox.set_row_spacing(1)
+            flowbox.set_column_spacing(1)
             flowbox.set_max_children_per_line(12)
-            flowbox_add_hover_support(flowbox)
+            if Gtk.get_major_version() == 3:
+                flowbox_add_hover_support(flowbox)
             flowbox.connect("child-activated", self._on_tool_activated)
 
             if Gtk.get_major_version() == 3:
@@ -118,23 +125,23 @@ class Toolbox(UIComponent):
                 )
                 flowbox.connect("drag-begin", _flowbox_drag_begin)
                 flowbox.connect("drag-data-get", _flowbox_drag_data_get)
-            else:
+            elif sys.platform != "darwin":
                 drag_source = Gtk.DragSource.new()
                 drag_source.connect("prepare", _flowbox_drag_prepare)
                 flowbox.add_controller(drag_source)
 
-            if Gtk.get_major_version() == 3:
-                expander.add(flowbox)
-            else:
-                expander.set_child(flowbox)
             for action_name, label, icon_name, shortcut, *rest in items:
                 button = create_toolbox_button(action_name, icon_name, label, shortcut)
                 flowbox.insert(button, -1)
 
             if Gtk.get_major_version() == 3:
+                revealer.add(flowbox)
                 toolbox.add(expander)
+                toolbox.add(revealer)
             else:
+                revealer.set_child(flowbox)
                 toolbox.append(expander)
+                toolbox.append(revealer)
 
         if Gtk.get_major_version() == 3:
             toolbox.show_all()
@@ -171,23 +178,22 @@ class Toolbox(UIComponent):
     def flowboxes(self):
         if self._toolbox:
             for expander in iter_children(self._toolbox):
-                yield expander.get_child()
+                if isinstance(expander, Gtk.Revealer):
+                    yield expander.get_child()
 
     def select_tool(self, tool_name: str) -> None:
         for flowbox in self.flowboxes():
+            flowbox.unselect_all()
             for child in iter_children(flowbox):
                 if child.action_name == tool_name:
                     flowbox.select_child(child)
-                    for fb in self.flowboxes():
-                        if fb is not flowbox:
-                            fb.unselect_all()
                     self.event_manager.handle(ToolSelected(tool_name))
-                    return
 
     @event_handler(ToolCompleted)
-    def _on_diagram_item_placed(self, event: TransactionCommit) -> None:
+    def _on_diagram_item_placed(self, event) -> None:
         if self.properties.get("reset-tool-after-create", True):
-            self.select_tool("toolbox-pointer")
+            # Select tool from an idle handler, so the original tool can complete properly.
+            GLib.idle_add(self.select_tool, "toolbox-pointer")
 
     @event_handler(ModelingLanguageChanged)
     def _on_modeling_language_changed(self, event: ModelingLanguageChanged) -> None:
@@ -210,7 +216,9 @@ class Toolbox(UIComponent):
         self._current_diagram_type = event.diagram and event.diagram.diagramType or ""
 
         expanded = self.expanded_sections()
-        for index, expander in enumerate(iter_children(self._toolbox)):
+        for index, expander in enumerate(
+            c for c in iter_children(self._toolbox) if isinstance(c, Gtk.Expander)
+        ):
             expander.set_property("expanded", expanded[index])
 
     def _on_tool_activated(self, flowbox, child):
@@ -306,8 +314,10 @@ else:
         )
         source.set_icon(theme_icon, 0, 0)
 
-        v = GObject.Value(GObject.TYPE_STRING)
-        v.set_string(child.action_name)
+        v = GObject.Value(
+            ToolboxActionDragData.__gtype__,
+            ToolboxActionDragData(action=child.action_name),
+        )
         return Gdk.ContentProvider.new_for_value(v)
 
 
